/*	RCT3 File Importer
	Copyright 2005 Jonathan Wilson

	This file is part of the RCT3 File Importer
	The RCT3 File Importer Program is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.

	Small modifications made by Klinn - May 2005 - search for 'by KLN'
	1) Added crude support for recolorable objects:
			- controls added to IDD_CREATESCENERY dialog box
			- parameter added to SaveOVL function
		 Note that the object texture bitmap's colors will be replaced when recolored in-game:
		  - color 0 = not known, avoid use
		  - colors 1-85 = re-color #1, shades from light to dark
			- colors 86-170 = re-color #2, light to dark
			- colors 171-255 = re-color #3, light to dark
		 A suitable custom indexed color table should be created and used when converting
		 an RGB-format image to indexed 256-color mode, so that the results can be more
		 easily visualized. Frontier appears to have used shades of blue for 1-85, magenta
		 for 86-170, and yellow for 171-255. Suggest leaving color 0 = white.
		 The interface really should be a combo box with possible options listed, rather
		 than a simple edit box where the user has to enter the correct code. Sigh.
	2) Added confirmation MessageBox after theme is successfully installed.
	3) Revised the GUISkinItemPos struct, swapped the 'Left' and 'Top' members, and
	   swapped the 'Right' and 'Bottom' members, so that game will pick up correct coords.
*/
#include <windows.h>
#include <d3d9types.h>
#include <stdio.h>
#include <queue>
#include <vector>
#include "libase.h"
#include <float.h>
#include <commdlg.h>
#include <shlobj.h>
#include "resource.h"
bool save = false;
unsigned long log2(unsigned long y)
{
	unsigned long result = 0;
	while(y)
	{
		result++;
		y >>= 1;
	}
	return result-1;
}
struct VERTEX {
	D3DVECTOR position;
	D3DVECTOR normal;
	D3DCOLOR color;
	FLOAT tu, tv;
};
struct SceneryItemVisualLOD
{
	unsigned long MeshType; //seems to be flag for what type of mesh this is, 0 = StaticShape, 3 = BoneShape
	char *LODName;
	unsigned char *Mesh; //either a StaticShape or a BoneShape, set to 0 in disk files
	unsigned long unk2; //seen 0 so far
	unsigned long unk3; //seen 0 so far
	unsigned long unk4; //seen 0 so far
	unsigned long unk5; //seen 0 so far
	unsigned long unk6; //seen 0 so far
	float unk7; //seen 1.0 so far
	float unk8; //seen 1.0 so far
	unsigned long unk9; //seen 0 so far
	float unk10; //seen 1.0 so far
	unsigned long unk11; //seen 0 so far
	float unk12; //seen 1.0 so far
	float distance; //has something to do with the distance this LOD model kicks in at or stops being used at 40 = high, 100 = med, 300 = low, 4000 = very low
	unsigned long ExtraDataPresent; //seen 0 and 1 so far, 1 seems to mean that there is data in the pointer below
	unsigned long unk14; //seen 0 so far
	unsigned long ExtraData; //either 0 or points to something, values are seen in animated/bone shape models only...
};
struct SceneryItemVisual
{
	unsigned long unk1; //seen 0 so far
	unsigned long unk2; //seen 0 so far
	float unk3; //seen 1.0 so far
	float unk4; //seen 1.0 so far
	unsigned long unk5; //seen 0 so far
	unsigned long LODCount;
	SceneryItemVisualLOD **LODMeshes;
	unsigned long unk6; //seen 0 so far
	unsigned long unk7; //seen 0 so far
	unsigned long unk8; //seen 0 so far
	unsigned long unk9; //seen 0 so far
	unsigned long unk10; //seen 0 so far
	unsigned long unk11; //seen 0 so far
};
struct FlicStruct
{
	unsigned long *FlicDataPtr; //always 0 on disk files
	unsigned long unk1; //always 1
	float unk2; //always 1.0
};
struct FlicHeader
{
	unsigned long Format;
	unsigned long Width;
	unsigned long Height;
	unsigned long Mipcount;
};
struct FlicMipHeader
{
	unsigned long MWidth;
	unsigned long MHeight;
	unsigned long Pitch;
	unsigned long Blocks;
};
struct Text
{
	char *name;
	char *text;
};
struct Icon
{
	char *name;
	unsigned long top;
	unsigned long left;
	unsigned long bottom;
	unsigned long right;
	char *texture;
};
struct IconTexture
{
	char *name;
	char *filename;
};
struct Scenery
{
	char *ovl;
	char *wallicon;
	char *icon;
	char *location;
	long cost;
	long refund;
	char *wallname;
	char *name;
	D3DVECTOR position;
	unsigned long type;
	BOOL flag1;
	BOOL flag2;
	BOOL flag3;
	BOOL flag4;
	BOOL flag5;
	BOOL flag6;
	BOOL flag7;
	BOOL flag8;
	BOOL flag9;
	BOOL flag10;
	BOOL flag11;
	BOOL flag12;
	BOOL flag13;
	BOOL flag14;
	BOOL flag15;
	BOOL flag16;
	BOOL flag17;
	BOOL flag18;
	BOOL flag19;
	BOOL flag20;
	BOOL flag21;
	BOOL flag22;
	BOOL flag23;
	BOOL flag24;
	BOOL flag25;
	BOOL flag26;
	BOOL flag27;
	BOOL flag28;
	BOOL flag29;
	BOOL flag30;
	BOOL flag31;
	BOOL flag32;
	unsigned long unk4;
	D3DVECTOR size;
	unsigned long supportstype;
	unsigned long unk17;
	unsigned long sizex;
	unsigned long sizey;
	unsigned long sizeflag;
	unsigned long extra;
	unsigned long unk31;
	unsigned long unk32;
	unsigned long unk33;
	BOOL flag33;
	BOOL flag34;
	BOOL flag35;
	BOOL flag36;
	BOOL flag37;
	BOOL flag38;
	BOOL flag39;
	BOOL flag40;
	BOOL flag41;
	BOOL flag42;
	BOOL flag43;
	BOOL flag44;
	BOOL flag45;
	BOOL flag46;
	BOOL flag47;
	BOOL flag48;
	BOOL flag49;
	BOOL flag50;
	BOOL flag51;
	BOOL flag52;
	BOOL flag53;
	BOOL flag54;
	BOOL flag55;
	BOOL flag56;
	BOOL flag57;
	BOOL flag58;
	BOOL flag59;
	BOOL flag60;
	BOOL flag61;
	BOOL flag62;
	BOOL flag63;
	BOOL flag64;
	long dunk2;
	unsigned long dunk3;
	long dunk4;
	unsigned long dunk5;
};
long CurrentTextString = -1;
long CurrentIcon = -1;
long CurrentIconTexture = -1;
long CurrentScenery = -1;
std::vector <Text *> TextStrings;
std::vector <Text *>::iterator TextStringsIterator;
std::vector <Icon *> Icons;
std::vector <Icon *>::iterator IconsIterator;
std::vector <IconTexture *> IconTextures;
std::vector <IconTexture *>::iterator IconTexturesIterator;
std::vector <Scenery *> SceneryItems;
std::vector <Scenery *>::iterator SceneryItemsIterator;
struct TextureStruct;
struct TextureStruct2
{
	TextureStruct *Texture; //points back to the TextureStruct
	FlicStruct *Flic; //points to the FlicStruct for this item
};
struct TextureStruct
{
	unsigned long unk1; //always 0x70007
	unsigned long unk2; //always 0x70007
	unsigned long unk3; //always 0x70007
	unsigned long unk4; //always 0x70007
	unsigned long unk5; //always 0x70007
	unsigned long unk6; //always 0x70007
	unsigned long unk7; //always 0x70007
	unsigned long unk8; //always 0x70007
	unsigned long unk9; //always 1
	unsigned long unk10; //always 8
	unsigned long unk11; //seen 0x0D and 0x10, use 0x10 for icons
	unsigned long *TextureData; //always 0 on disk files, use GUIIcon:txs as a symbol resolve for icons
	unsigned long unk12; //always 1
	FlicStruct **Flic; //always points to pointer before flic data
	TextureStruct2 *ts2;
};
struct Tex
{
	TextureStruct t1;
	TextureStruct2 t2;
};
struct Flic
{
	FlicStruct *fl;
	FlicStruct fl2;
};
struct GUISkinItemPos
{
	unsigned long left; //Modded by KLN - swapped left & top, swapped right & bottom
	unsigned long top;
	unsigned long right;
	unsigned long bottom;
};
struct GUISkinItem
{
	unsigned long unk1; //0 on disk files
	TextureStruct *tex; //0 on disk files
	GUISkinItemPos *pos;
	unsigned long unk2; //0 on disk files
};
struct GSI
{
	GUISkinItem g1;
	GUISkinItemPos g2;
};
struct SceneryParams
{
	char *type;
	char *params;
};
char *stdstyle = "Style\\Themed\\";
char *stdstrings = "StyleIndex:int\0GUIIcon:txs\0GuestInjectionPoint\0LitterBin\0 Capacity 4.0\0ParkBench\0ParkEntrance\0QueueLineEntertainer\0RideEvent\0Sign\0 EnableNoEntry\0VistaPoint\0Waterable\0 Capacity 300.0\0adventure\0crazygolf\0crazygolfholes\0girder\0ornate\0pathasphalt\0pathcrazy\0pathdirt\0pathleafy\0marble\0pathtarmac\0flatqueueset1\0romanpillar\0scaffolding\0space\0spooky\0steel\0ts1\0ts2\0ts3\0ts4\0ts5\0flatwildwest\0\0MapColourBlue:flt\0MapColourRed:flt\0MapColorGreen:flt";
/*
valid values for "type" I have found so far are:
GuestInjectionPoint
LitterBin (params is Capacity 4.0, dont know if changing it will work)
ParkBench
ParkEntrance
QueueLineEntertainer (for the queueline tv)
RideEvent (params may be a string like doom,shark,truck)
Sign (params can be EnableNoEntry or EnableNoEntry QueueSign)
VistaPoint
Waterable (for the flowers, params is Capacity 300.0, dont know if changing it will work)
*/
struct SceneryItemData //set all to 0 and see what happens
{
	unsigned long unkf33:1; //might have something to do with whether a scenery object has 0 clearence or not
	unsigned long unkf34:1;
	unsigned long unkf35:1;
	unsigned long unkf36:1;
	unsigned long unkf37:1;
	unsigned long unkf38:1;
	unsigned long unkf39:1;
	unsigned long unkf40:1;
	unsigned long unkf41:1;
	unsigned long unkf42:1;
	unsigned long unkf43:1;
	unsigned long unkf44:1;
	unsigned long unkf45:1;
	unsigned long unkf46:1;
	unsigned long unkf47:1;
	unsigned long unkf48:1;
	unsigned long unkf49:1;
	unsigned long unkf50:1;
	unsigned long unkf51:1;
	unsigned long unkf52:1;
	unsigned long unkf53:1;
	unsigned long unkf54:1;
	unsigned long unkf55:1;
	unsigned long unkf56:1;
	unsigned long unkf57:1;
	unsigned long unkf58:1;
	unsigned long unkf59:1;
	unsigned long unkf60:1;
	unsigned long unkf61:1;
	unsigned long unkf62:1;
	unsigned long unkf63:1;
	unsigned long unkf64:1;
	long unk2;
	unsigned long unk3;
	long *unk4;
	unsigned long unk5;
};
#define TYPE_TREE 0
#define TYPE_PLANT 1
#define TYPE_SHRUB 2
#define TYPE_FLOWERS 3
#define TYPE_FENCE 4
#define TYPE_WALL_MISC 5
#define TYPE_PATH_LAMP 6
#define TYPE_SCENERY_SMALL 7
#define TYPE_SCENERY_MEDIUM 8
#define TYPE_SCENERY_LARGE 9
#define TYPE_SCENERY_ANAMATRONIC 10
#define TYPE_SCENERY_MISC 11
#define TYPE_SUPPORT_MIDDLE 12
#define TYPE_SUPPORT_TOP 13
#define TYPE_SUPPORT_BOTTOM 14
#define TYPE_SUPPORT_BOTTOM_EXTRA 15
#define TYPE_SUPPORT_GIRDER 16
#define TYPE_SUPPORT_CAP 17
#define TYPE_RIDE_TRACK 18
#define TYPE_PATH 19
#define TYPE_PARK_ENTRANCE 20
#define TYPE_LITTER 21
#define TYPE_GUEST_INJECT 22
#define TYPE_RIDE 23
#define TYPE_RIDE_ENTRANCE 24
#define TYPE_RIDE_EXIT 25
#define TYPE_KEEP_CLEAR_FENCE 26
#define TYPE_STALL 27
#define TYPE_RIDE_EVENT 28
#define TYPE_FIREWORK 29
#define TYPE_LITTER_BIN 30
#define TYPE_BENCH 31
#define TYPE_SIGN 32
#define TYPE_PHOTO_POINT 33
#define TYPE_WALL_STRAIGHT 34
#define TYPE_WALL_ROOF 35
#define TYPE_WALL_CORNER 36

#define SIZE_FULLTILE 0
#define SIZE_PATHEDGEINNER 1
#define SIZE_PATHEDGEOUTER 2
#define SIZE_WALL 3
#define SIZE_QUARTER 4
#define SIZE_HALF 5
#define SIZE_PATHCENTER 6
#define SIZE_CORNER 7
#define SIZE_PATHEDGEJOIN 8

struct SceneryItem
{
	unsigned long unk1; //is 0 on disk files
	//flags
	unsigned long unkf1:1;
	unsigned long unkf2:1; //not used anywhere that I can see
	unsigned long unkf3:1; 
	unsigned long unkf4:1; //on for vertical ride bases, stalls, flats and some other stuff
	unsigned long unkf5:1; //not used anywhere that I can see
	unsigned long unkf6:1; //is set to 1 for all tracks except vertical rides (i.e. elevator etc)
	unsigned long unkf7:1;
	unsigned long unkf8:1;
	unsigned long unkf9:1; //set only on the column courtyard
	unsigned long unkf10:1; //set for some track pieces
	unsigned long unkf11:1; //set for some track pieces
	unsigned long unkf12:1; //set for some track pieces
	unsigned long unkf13:1; //controls whether an object can have walls on the same square as it except when its a 1/2 tile scenery item
	unsigned long unkf14:1;
	unsigned long unkf15:1; //set for some support pieces
	unsigned long unkf16:1; //set for some support pieces
	unsigned long unkf17:1; //set for water track (i.e. jetskis etc)
	unsigned long unkf18:1; //not used anywhere that I can see
	unsigned long unkf19:1; //not used anywhere that I can see
	unsigned long unkf20:1; //set for helter-skelters
	unsigned long unkf21:1; //not used anywhere that I can see
	unsigned long unkf22:1; //not used anywhere that I can see
	unsigned long unkf23:1; //not used anywhere that I can see
	unsigned long unkf24:1; //not used anywhere that I can see
	unsigned long unkf25:1; //not used anywhere that I can see
	unsigned long unkf26:1; //not used anywhere that I can see
	unsigned long unkf27:1; //not used anywhere that I can see
	unsigned long unkf28:1; //not used anywhere that I can see
	unsigned long unkf29:1; //not used anywhere that I can see
	unsigned long unkf30:1; //not used anywhere that I can see
	unsigned long unkf31:1; //not used anywhere that I can see
	unsigned long unkf32:1; //not used anywhere that I can see
	unsigned long size;
	unsigned long unk4; //valid values are 0-17
	unsigned long xsquares;
	unsigned long ysquares;
	SceneryItemData *data;
	float xpos; //position?
	float ypos; //position?
	float zpos; //position?
	float xsize;
	float ysize;
	float zsize;
	unsigned long unk13; //is 0 except in track parts
	unsigned long unk14; //is 0 except in track parts (pointer I think)
	long cost;
	long refund;
	unsigned long unk17; //is 0,4,5 4 is found on the queue paths
	unsigned long type;
	char *supports;
	unsigned long svdcount;
	SceneryItemVisual **svd;
	GUISkinItem *gsi;
	GUISkinItem *wallsetgsi;
	wchar_t *wallsetname;
	char *OvlName;
	unsigned long count;
	SceneryParams *params;
	unsigned long unk25; //is 0 except in flat rides
	unsigned long unk26; //is 0 except in flat rides (pointer I think)
	wchar_t *Name;
	unsigned long unk27; //is 0
	unsigned long unk28; //is 0
	unsigned long unk29; //is 0 except in stalls
	unsigned long unk30; //is 0 except in stalls
	unsigned long unk31; //can have values, is usually 0 though
	unsigned long unk32; //can have values, is usually 0 though
	unsigned long unk33; //can have values, is usually 0 though
	unsigned long unk34; //is 0
	unsigned long unk35; //is 0
	unsigned long unk36; //is 0
	unsigned long unk37; //is 0
	unsigned long unk38; //is 0
	unsigned long unk39; //is 0
	unsigned long unk40; //is 0
	unsigned long unk41; //is 0
	unsigned long unk42; //is 0 except for flying saucers and dodgems
	unsigned long unk43; //is 0 except for flying saucers and dodgems (pointer I think)
	unsigned long unk44; //is 0
	unsigned long unk45; //is 0 except for trampoline
	unsigned long unk46; //is 0 except for trampoline
	unsigned long unk47; //is 0 except for trampoline
	unsigned long unk48; //is 0 except for trampoline
};
struct FlexiTextureStruct
{
	unsigned long scale;
	unsigned long width;
	unsigned long height;
	unsigned long Recolorable; //1 = first color recolorable, 2 = second color recolorable, 4 = third color recolorable, combination of them = multiple colors recolorable
	unsigned char *palette;
	unsigned char *texture;
	unsigned char *alpha;
};
struct FlexiTextureInfoStruct
{
	unsigned long scale;
	unsigned long width;
	unsigned long height;
	unsigned long unk1;
	unsigned long Recolorable; //1 = first color recolorable, 2 = second color recolorable, 4 = third color recolorable, combination of them = multiple colors recolorable
	unsigned long unk3;
	unsigned long *offset1;
	unsigned long unk4;
	FlexiTextureStruct *fts2;
};
struct StaticShape2 {
	unsigned long unk1; //always 0xFFFFFFFFF
	FlexiTextureInfoStruct *fts; //is 0 in disk files
	unsigned long *TextureData; //is 0 in disk files
	unsigned long PlaceTexturing; //0 = dont see texturing on things when being placed, 1 = see texturing on things when being placed
	unsigned long textureflags; //always 0
	unsigned long unk4; //seen values of 3 or 1 for this, 3 is more common
	unsigned long VertexCount;
	unsigned long TriangleCount;
	VERTEX *Vertexes;
	unsigned long *Triangles;
};
struct StaticShape3 { //set to 0 and see what happens
	float unk1;
	float unk2;
	float unk3;
	float unk4;
	float unk5;
	float unk6;
	float unk7;
	float unk8;
	float unk9;
	float unk10;
	float unk11;
	float unk12;
	float unk13;
	float unk14;
	float unk15;
	float unk16;
};
struct StaticShape1 {
	D3DVECTOR BoundingBox1;
	D3DVECTOR BoundingBox2;
	unsigned long TotalVertexCount;
	unsigned long TotalTriangleCount;
	unsigned long MeshCount2;
	unsigned long MeshCount;
	StaticShape2 **sh;
	unsigned long SubModelCount; //dont know about this
	StaticShape3 *data; //dont know about this, setting to all 0 seemed to have no effect that I could see
	char *name;
};
struct OvlHeader {
	unsigned long magic;
	unsigned long reserved;
	unsigned long version;
	unsigned long References;
};
struct OvlHeader2 {
	unsigned long unk;
	unsigned long FileTypeCount;
};
struct File
{
	unsigned long size;
	unsigned long reloffset;
	unsigned long *data;
	unsigned long unk;
};
struct FileType
{
	unsigned long count;
	File *dataptr;
	unsigned long reloffset;
	unsigned long size;
};
struct SymbolStruct
{
	char *Symbol;
	unsigned long *data;
	unsigned long unk;
};
struct LoaderStruct
{
	unsigned long LoaderType;
	unsigned long *data;
	unsigned long unk1;
	SymbolStruct *Sym;
	unsigned long SymbolsToResolve;
};
struct SymbolRefStruct
{
	unsigned long *reference;
	char *Symbol;
	LoaderStruct *ldr;
};
struct OutOvlFile
{
	char FileName[MAX_PATH];
	FileType Types[9];
};
unsigned long OutOvlCount = 0;
OutOvlFile OutOvlFiles[2];
File Type0Files[2];
File Type2Files[6];
std::queue<unsigned long *> relocations;
std::queue<unsigned long> fixups;
BITMAPFILEHEADER bmf;
BITMAPINFOHEADER bmi;
unsigned long ShapeSize;
unsigned long DoRelocation(unsigned long *reloc)
{
	for (unsigned long i = 0;i < 9;i++)
	{
		for (unsigned long j = 0; j < OutOvlFiles[OutOvlCount].Types[i].count;j++)
		{
			if ((reloc >= (unsigned long *)OutOvlFiles[OutOvlCount].Types[i].dataptr[j].data) && (reloc <= (unsigned long *)(((unsigned char *)OutOvlFiles[OutOvlCount].Types[i].dataptr[j].data)+OutOvlFiles[OutOvlCount].Types[i].dataptr[j].size)))
			{
				return ((unsigned long)reloc - (unsigned long)(OutOvlFiles[OutOvlCount].Types[i].dataptr[j].data)+(unsigned long)(OutOvlFiles[OutOvlCount].Types[i].dataptr[j].reloffset));
			}
		}
	}
	for (unsigned long i = 0;i < 9;i++)
	{
		for (unsigned long j = 0; j < OutOvlFiles[OutOvlCount-1].Types[i].count;j++)
		{
			if ((reloc >= (unsigned long *)OutOvlFiles[OutOvlCount-1].Types[i].dataptr[j].data) && (reloc <= (unsigned long *)(((unsigned char *)OutOvlFiles[OutOvlCount-1].Types[i].dataptr[j].data)+OutOvlFiles[OutOvlCount-1].Types[i].dataptr[j].size)))
			{
				return ((unsigned long)reloc - (unsigned long)(OutOvlFiles[OutOvlCount-1].Types[i].dataptr[j].data)+(unsigned long)(OutOvlFiles[OutOvlCount-1].Types[i].dataptr[j].reloffset));
			}
		}
	}
	return 0xFFFFFFFF;
}
HINSTANCE hInst;
HWND SceneryDlgHwnd;
unsigned char *data;
bool SaveOvl(char *model,char *texture,char *name,char *filename,char *txs,char *recolor) 
//modded by KLN: added recolor parameter
{
	SymbolRefStruct *Ref;
	unsigned long SymCount = 3;
	unsigned long LdrCount = 3;
	unsigned long SymRefCount = 5;
	SymbolStruct *Sym;
	LoaderStruct *Ldr;
	unsigned char *texturedata;
	unsigned long *ofs1;
	RGBQUAD colors[256];
	RGBQUAD *c;
	FILE *f;
	unsigned long namelength = strlen(name)+1;
	unsigned long txslength = strlen(txs)+1;
	unsigned long length = namelength;
	length += namelength + 4;
	length += namelength + 4;
	length += namelength + 4;
	length += txslength + 4;
	char *names = new char[length];
	memset(names,0,length);
	char *objname = names;
	char *texturename = objname + namelength;
	char *shapename = texturename + namelength + 4;
	char *sceneryname = shapename + namelength + 4;
	char *txsname = sceneryname + namelength + 4;
	sprintf(objname,"%s",name);
	sprintf(texturename,"%s:ftx",name);
	sprintf(shapename,"%s:shs",name);
	sprintf(sceneryname,"%s:svd",name);
	sprintf(txsname,"%s:txs",txs);
	unsigned long scenerysize = sizeof(SceneryItemVisual)+(sizeof(SceneryItemVisualLOD)*3)+12;
	unsigned char *svd = new unsigned char[scenerysize];
	memset(svd,0,scenerysize);
	SceneryItemVisual *sv = (SceneryItemVisual *)svd;
	SceneryItemVisualLOD **svlod = (SceneryItemVisualLOD **)(svd+sizeof(SceneryItemVisual));
	SceneryItemVisualLOD *svlod1 = (SceneryItemVisualLOD *)(svd+sizeof(SceneryItemVisual)+12);
	SceneryItemVisualLOD *svlod2 = (SceneryItemVisualLOD *)(svd+sizeof(SceneryItemVisual)+12+sizeof(SceneryItemVisualLOD));
	SceneryItemVisualLOD *svlod3 = (SceneryItemVisualLOD *)(svd+sizeof(SceneryItemVisual)+12+(sizeof(SceneryItemVisualLOD)*2));
	sv->unk1 = 0;
	sv->unk2 = 0;
	sv->unk3 = 1.0;
	sv->unk4 = 1.0;
	sv->unk5 = 0;
	sv->LODCount = 3;
	sv->LODMeshes = svlod;
	sv->unk6 = 0;
	sv->unk7 = 0;
	sv->unk8 = 0;
	sv->unk9 = 0;
	sv->unk10 = 0;
	sv->unk11 = 0;
	svlod[0] = svlod1;
	svlod[1] = svlod2;
	svlod[2] = svlod3;
	svlod1->MeshType = 0;
	svlod1->LODName = objname;
	svlod1->Mesh = 0;
	svlod1->unk2 = 0;
	svlod1->unk3 = 0;
	svlod1->unk4 = 0;
	svlod1->unk5 = 0;
	svlod1->unk6 = 0;
	svlod1->unk7 = 1.0;
	svlod1->unk8 = 1.0;
	svlod1->unk9 = 0;
	svlod1->unk10 = 1.0;
	svlod1->unk11 = 0;
	svlod1->unk12 = 1.0;
	svlod1->distance = 40;
	svlod1->ExtraDataPresent = 0;
	svlod1->unk14 = 0;
	svlod1->ExtraData = 0;
	svlod2->MeshType = 0;
	svlod2->LODName = objname;
	svlod2->Mesh = 0;
	svlod2->unk2 = 0;
	svlod2->unk3 = 0;
	svlod2->unk4 = 0;
	svlod2->unk5 = 0;
	svlod2->unk6 = 0;
	svlod2->unk7 = 1.0;
	svlod2->unk8 = 1.0;
	svlod2->unk9 = 0;
	svlod2->unk10 = 1.0;
	svlod2->unk11 = 0;
	svlod2->unk12 = 1.0;
	svlod2->distance = 100;
	svlod2->ExtraDataPresent = 0;
	svlod2->unk14 = 0;
	svlod2->ExtraData = 0;
	svlod3->MeshType = 0;
	svlod3->LODName = objname;
	svlod3->Mesh = 0;
	svlod3->unk2 = 0;
	svlod3->unk3 = 0;
	svlod3->unk4 = 0;
	svlod3->unk5 = 0;
	svlod3->unk6 = 0;
	svlod3->unk7 = 1.0;
	svlod3->unk8 = 1.0;
	svlod3->unk9 = 0;
	svlod3->unk10 = 1.0;
	svlod3->unk11 = 0;
	svlod3->unk12 = 1.0;
	svlod3->distance = 4000;
	svlod3->ExtraDataPresent = 0;
	svlod3->unk14 = 0;
	svlod3->ExtraData = 0;
	ASE_Scene *scene = ASE_loadFilename(model);
	if (scene->objectCount != 1)
	{
		MessageBox(SceneryDlgHwnd,"Multiple objects, cant convert mesh","Error",MB_OK);
		return false;
	}
	ASE_Mesh *obj = &scene->objs->mesh;
	ShapeSize = sizeof(StaticShape1)+sizeof(StaticShape2)+sizeof(StaticShape3)+(obj->vertexCount*sizeof(VERTEX)) + ((obj->faceCount*3)*sizeof(unsigned long))+4;
	unsigned char *shape = new unsigned char[ShapeSize];
	memset(shape,0,ShapeSize);
	StaticShape1 *sh = (StaticShape1 *)shape;
	StaticShape2 **sh2x = (StaticShape2 **)(shape + sizeof(StaticShape1));
	StaticShape2 *sh2 = (StaticShape2 *)(shape + sizeof(StaticShape1) + 4);
	StaticShape3 *sh3 = (StaticShape3 *)(shape + sizeof(StaticShape1) + sizeof(StaticShape2) + 4);
	VERTEX *Vertexes = (VERTEX *)(shape + sizeof(StaticShape1) + sizeof(StaticShape2) + sizeof(StaticShape3) + 4);
	unsigned long *Triangles = (unsigned long *)(shape + sizeof(StaticShape1) + sizeof(StaticShape2) + sizeof(StaticShape3) + (obj->vertexCount*sizeof(VERTEX)) + 4);
	sh->BoundingBox2.x = FLT_MIN;
	sh->BoundingBox2.y = FLT_MIN;
	sh->BoundingBox2.z = FLT_MIN;
	sh->BoundingBox1.x = FLT_MAX;
	sh->BoundingBox1.y = FLT_MAX;
	sh->BoundingBox1.z = FLT_MAX;
	for (long i = 0;i < obj->vertexCount;i++)
	{
		if (obj->vertices[i].x < sh->BoundingBox1.x)
		{
			sh->BoundingBox1.x = obj->vertices[i].x;
		}
		if (obj->vertices[i].y < sh->BoundingBox1.y)
		{
			sh->BoundingBox1.y = obj->vertices[i].y;
		}
		if (obj->vertices[i].z < sh->BoundingBox1.z)
		{
			sh->BoundingBox1.z = obj->vertices[i].z;
		}
		if (obj->vertices[i].x > sh->BoundingBox2.x)
		{
			sh->BoundingBox2.x = obj->vertices[i].x;
		}
		if (obj->vertices[i].y > sh->BoundingBox2.y)
		{
			sh->BoundingBox2.y = obj->vertices[i].y;
		}
		if (obj->vertices[i].z > sh->BoundingBox2.z)
		{
			sh->BoundingBox2.z = obj->vertices[i].z;
		}
		Vertexes[i].position.x = obj->vertices[i].x;
		Vertexes[i].position.y = obj->vertices[i].y;
		Vertexes[i].position.z = obj->vertices[i].z;
		Vertexes[i].color = 0xFFFFFFFF;
		Vertexes[i].normal.x = obj->vertex_normals[i].x;
		Vertexes[i].normal.y = obj->vertex_normals[i].y;
		Vertexes[i].normal.z = obj->vertex_normals[i].z;
		Vertexes[i].tu = 0;
		Vertexes[i].tv = 0;
	}
	for (i = 0;i < obj->faceCount*3;i+=3)
	{
		Triangles[i] = obj->faces[i/3].vertex[0];
		Triangles[i+1] = obj->faces[i/3].vertex[1];
		Triangles[i+2] = obj->faces[i/3].vertex[2];
		Vertexes[obj->faces[i/3].vertex[0]].tu = obj->texture_coordinates[obj->faces[i/3].texture_coordinates[0]].x;
		Vertexes[obj->faces[i/3].vertex[0]].tv = obj->texture_coordinates[obj->faces[i/3].texture_coordinates[0]].y;
		Vertexes[obj->faces[i/3].vertex[1]].tu = obj->texture_coordinates[obj->faces[i/3].texture_coordinates[1]].x;
		Vertexes[obj->faces[i/3].vertex[1]].tv = obj->texture_coordinates[obj->faces[i/3].texture_coordinates[1]].y;
		Vertexes[obj->faces[i/3].vertex[2]].tu = obj->texture_coordinates[obj->faces[i/3].texture_coordinates[2]].x;
		Vertexes[obj->faces[i/3].vertex[2]].tv = obj->texture_coordinates[obj->faces[i/3].texture_coordinates[2]].y;
	}
	sh->TotalVertexCount = obj->vertexCount;
	sh->TotalTriangleCount = obj->faceCount*3;
	sh->MeshCount = 1;
	sh->MeshCount2 = 1;
	sh->sh = sh2x;
	sh->SubModelCount = 0;
	//sh->data = sh3;
	sh->data = 0;
	sh->name = objname;
	sh3->unk1 = 0;
	sh3->unk2 = 0;
	sh3->unk3 = 0;
	sh3->unk4 = 0;
	sh3->unk5 = 0;
	sh3->unk6 = 0;
	sh3->unk7 = 0;
	sh3->unk8 = 0;
	sh3->unk9 = 0;
	sh3->unk10 = 0;
	sh3->unk11 = 0;
	sh3->unk12 = 0;
	sh3->unk13 = 0;
	sh3->unk14 = 0;
	sh3->unk15 = 0;
	sh3->unk16 = 0;
	*sh2x = sh2;
	sh2->unk1 = 0xFFFFFFFF;
	sh2->fts = 0;
	sh2->TextureData = 0;
	sh2->PlaceTexturing = 0;
	sh2->textureflags = 0;
	sh2->unk4 = 3;
	sh2->VertexCount = obj->vertexCount;
	sh2->TriangleCount = obj->faceCount*3;
	sh2->Vertexes = Vertexes;
	sh2->Triangles = Triangles;
	f = fopen(texture,"rb");
	if (f == 0)
	{
		MessageBox(SceneryDlgHwnd,"Texture not found","Error",MB_OK);
		return false;
	}
	fread(&bmf,sizeof(bmf),1,f);
	fread(&bmi,sizeof(bmi),1,f);
	if ((bmi.biWidth != bmi.biHeight) || (bmi.biBitCount != 8) || (bmi.biWidth % 2))
	{
		MessageBox(SceneryDlgHwnd,"Texture not square 8bpp image with height/width a power of 2","Error",MB_OK);
		return false;
	}
	fread(colors,sizeof(colors),1,f);
	data = new unsigned char[bmi.biWidth * bmi.biHeight];
	memset(data,0,bmi.biWidth * bmi.biHeight);
	unsigned char *temp;
	for (long i = bmi.biHeight;i > 0;i--)
	{
		temp = data + ((i-1)*bmi.biWidth);
		fread(temp,bmi.biWidth,1,f);
	}
	fclose(f);
	texturedata = new unsigned char[sizeof(colors)+sizeof(FlexiTextureInfoStruct)+sizeof(FlexiTextureStruct)+4];
	memset(texturedata,0,sizeof(colors)+sizeof(FlexiTextureInfoStruct)+sizeof(FlexiTextureStruct)+4);
	FlexiTextureInfoStruct *fti = (FlexiTextureInfoStruct *)texturedata;
	FlexiTextureStruct *fts = (FlexiTextureStruct *)(texturedata+sizeof(FlexiTextureInfoStruct)+4);
	c = (RGBQUAD *)(texturedata+sizeof(FlexiTextureInfoStruct)+sizeof(FlexiTextureStruct)+4);
	ofs1 = (unsigned long *)(texturedata+sizeof(FlexiTextureInfoStruct));
	memcpy(c,colors,sizeof(RGBQUAD)*256);
	fti->scale = log2(bmi.biWidth);
	fti->height = bmi.biHeight;
	fti->width = bmi.biWidth;
	fti->unk1 = 0;
	fti->Recolorable = atol( recolor ); //modded by KLN
	fti->unk3 = 1;
	fti->offset1 = ofs1;
	fti->unk4 = 1;
	fti->fts2 = fts;
	fts->scale = fti->scale;
	fts->width = fti->width;
	fts->height = fti->height;
	fts->Recolorable = fti->Recolorable;
	fts->palette = (unsigned char*)c;
	fts->texture = data;
	fts->alpha = 0;
	Sym = new SymbolStruct[sizeof(SymbolStruct)*SymCount];
	memset(Sym,0,sizeof(SymbolStruct)*SymCount);
	Sym[0].Symbol = texturename;
	Sym[0].data = (unsigned long *)texturedata;
	Sym[0].unk = 1;
	Sym[1].Symbol = shapename;
	Sym[1].data = (unsigned long *)shape;
	Sym[1].unk = 1;
	Sym[2].Symbol = sceneryname;
	Sym[2].data = (unsigned long *)svd;
	Sym[2].unk = 1;
	Ldr = new LoaderStruct[sizeof(LoaderStruct)*LdrCount];
	memset(Ldr,0,sizeof(LoaderStruct)*LdrCount);
	Ldr[0].LoaderType = 0;
	Ldr[0].data = (unsigned long *)texturedata;
	Ldr[0].unk1 = 0;
	Ldr[0].Sym = &Sym[0];
	Ldr[0].SymbolsToResolve = 0;
	Ldr[1].LoaderType = 1;
	Ldr[1].data = (unsigned long *)svd;
	Ldr[1].unk1 = 0;
	Ldr[1].Sym = &Sym[1];
	Ldr[1].SymbolsToResolve = 3;
	Ldr[2].LoaderType = 2;
	Ldr[2].data = (unsigned long *)shape;
	Ldr[2].unk1 = 0;
	Ldr[2].Sym = &Sym[2];
	Ldr[2].SymbolsToResolve = 2;
	Ref = new SymbolRefStruct[sizeof(SymbolRefStruct)*SymRefCount];
	memset(Ref,0,sizeof(SymbolRefStruct)*SymRefCount);
	Ref[0].ldr = &Ldr[1];
	Ref[0].Symbol = shapename;
	Ref[0].reference = (unsigned long *)&svlod1->Mesh;
	Ref[1].ldr = &Ldr[1];
	Ref[1].Symbol = shapename;
	Ref[1].reference = (unsigned long *)&svlod2->Mesh;
	Ref[2].ldr = &Ldr[1];
	Ref[2].Symbol = shapename;
	Ref[2].reference = (unsigned long *)&svlod3->Mesh;
	Ref[3].ldr = &Ldr[2];
	Ref[3].Symbol = texturename;
	Ref[3].reference = (unsigned long *)&sh2->fts;
	Ref[4].ldr = &Ldr[2];
	Ref[4].Symbol = txsname;
	Ref[4].reference = (unsigned long *)&sh2->TextureData;
	sprintf(OutOvlFiles[0].FileName,"%s.unique.ovl",filename);
	Type0Files[0].size = (unsigned long)length;
	Type0Files[0].reloffset = 0;
	Type0Files[0].data = (unsigned long *)names;
	Type0Files[0].unk = 0;
	Type0Files[1].size = bmi.biWidth * bmi.biHeight;
	Type0Files[1].reloffset = Type0Files[0].size + Type0Files[0].reloffset;
	Type0Files[1].data = (unsigned long *)data;
	Type0Files[1].unk = 0;
	Type2Files[0].size = sizeof(SymbolStruct)*SymCount;
	Type2Files[0].reloffset = Type0Files[1].size + Type0Files[1].reloffset;
	Type2Files[0].data = (unsigned long *)Sym;
	Type2Files[0].unk = 0;
	Type2Files[1].size = sizeof(LoaderStruct)*LdrCount;
	Type2Files[1].reloffset = Type2Files[0].size + Type2Files[0].reloffset;
	Type2Files[1].data = (unsigned long *)Ldr;
	Type2Files[1].unk = 0;
	Type2Files[2].size = sizeof(SymbolRefStruct)*SymRefCount;
	Type2Files[2].reloffset = Type2Files[1].size + Type2Files[1].reloffset;
	Type2Files[2].data = (unsigned long *)Ref;
	Type2Files[2].unk = 0;
	Type2Files[3].size = sizeof(colors)+sizeof(FlexiTextureInfoStruct)+sizeof(FlexiTextureStruct)+1;
	Type2Files[3].reloffset = Type2Files[2].size + Type2Files[2].reloffset;
	Type2Files[3].data = (unsigned long *)texturedata;
	Type2Files[3].unk = 0;
	Type2Files[4].size = scenerysize;
	Type2Files[4].reloffset = Type2Files[3].size + Type2Files[3].reloffset;
	Type2Files[4].data = (unsigned long *)svd;
	Type2Files[4].unk = 0;
	Type2Files[5].size = ShapeSize;
	Type2Files[5].reloffset = Type2Files[4].size + Type2Files[4].reloffset;
	Type2Files[5].data = (unsigned long *)shape;
	Type2Files[5].unk = 0;
	memset(OutOvlFiles[0].Types,0,sizeof(OutOvlFiles[0].Types));
	OutOvlFiles[0].Types[0].count = 2;
	OutOvlFiles[0].Types[0].dataptr = Type0Files;
	OutOvlFiles[0].Types[0].reloffset = 0;
	OutOvlFiles[0].Types[0].size = 0;
	for (unsigned long i = 0;i < OutOvlFiles[0].Types[0].count;i++)
	{
		OutOvlFiles[0].Types[0].size += Type0Files[i].size;
	}
	OutOvlFiles[0].Types[1].reloffset = OutOvlFiles[0].Types[0].size + OutOvlFiles[0].Types[0].reloffset;
	OutOvlFiles[0].Types[2].count = 6;
	OutOvlFiles[0].Types[2].dataptr = Type2Files;
	OutOvlFiles[0].Types[2].reloffset = OutOvlFiles[0].Types[1].size + OutOvlFiles[0].Types[1].reloffset;
	OutOvlFiles[0].Types[2].size = 0;
	for (i = 0;i < OutOvlFiles[0].Types[2].count;i++)
	{
		OutOvlFiles[0].Types[2].size += Type2Files[i].size;
	}
	OutOvlFiles[0].Types[3].reloffset = OutOvlFiles[0].Types[2].size + OutOvlFiles[0].Types[2].reloffset;
	OutOvlFiles[0].Types[4].reloffset = OutOvlFiles[0].Types[2].size + OutOvlFiles[0].Types[2].reloffset;
	OutOvlFiles[0].Types[5].reloffset = OutOvlFiles[0].Types[2].size + OutOvlFiles[0].Types[2].reloffset;
	OutOvlFiles[0].Types[6].reloffset = OutOvlFiles[0].Types[2].size + OutOvlFiles[0].Types[2].reloffset;
	OutOvlFiles[0].Types[7].reloffset = OutOvlFiles[0].Types[2].size + OutOvlFiles[0].Types[2].reloffset;
	OutOvlFiles[0].Types[8].reloffset = OutOvlFiles[0].Types[2].size + OutOvlFiles[0].Types[2].reloffset;
	relocations.push((unsigned long *)&Sym[0].Symbol);
	relocations.push((unsigned long *)&Sym[0].data);
	relocations.push((unsigned long *)&Sym[1].Symbol);
	relocations.push((unsigned long *)&Sym[1].data);
	relocations.push((unsigned long *)&Sym[2].Symbol);
	relocations.push((unsigned long *)&Sym[2].data);
	relocations.push((unsigned long *)&Ldr[0].data);
	relocations.push((unsigned long *)&Ldr[0].Sym);
	relocations.push((unsigned long *)&Ldr[1].data);
	relocations.push((unsigned long *)&Ldr[1].Sym);
	relocations.push((unsigned long *)&Ldr[2].data);
	relocations.push((unsigned long *)&Ldr[2].Sym);
	relocations.push((unsigned long *)&Ref[0].reference);
	relocations.push((unsigned long *)&Ref[0].Symbol);
	relocations.push((unsigned long *)&Ref[0].ldr);
	relocations.push((unsigned long *)&Ref[1].reference);
	relocations.push((unsigned long *)&Ref[1].Symbol);
	relocations.push((unsigned long *)&Ref[1].ldr);
	relocations.push((unsigned long *)&Ref[2].reference);
	relocations.push((unsigned long *)&Ref[2].Symbol);
	relocations.push((unsigned long *)&Ref[2].ldr);
	relocations.push((unsigned long *)&Ref[3].reference);
	relocations.push((unsigned long *)&Ref[3].Symbol);
	relocations.push((unsigned long *)&Ref[3].ldr);
	relocations.push((unsigned long *)&Ref[4].reference);
	relocations.push((unsigned long *)&Ref[4].Symbol);
	relocations.push((unsigned long *)&Ref[4].ldr);
	relocations.push((unsigned long *)&fti->offset1);
	relocations.push((unsigned long *)&fti->fts2);
	relocations.push((unsigned long *)&fts->palette);
	relocations.push((unsigned long *)&fts->texture);
	relocations.push((unsigned long *)&fts->alpha);
	relocations.push((unsigned long *)&sv->LODMeshes);
	relocations.push((unsigned long *)&svlod[0]);
	relocations.push((unsigned long *)&svlod[1]);
	relocations.push((unsigned long *)&svlod[2]);
	relocations.push((unsigned long *)&svlod1->LODName);
	relocations.push((unsigned long *)&svlod2->LODName);
	relocations.push((unsigned long *)&svlod3->LODName);
	relocations.push((unsigned long *)&sh->sh);
	relocations.push((unsigned long *)&sh->data);
	relocations.push((unsigned long *)&sh->name);
	relocations.push((unsigned long *)sh->sh);
	relocations.push((unsigned long *)&sh2->Vertexes);
	relocations.push((unsigned long *)&sh2->Triangles);
	while (relocations.empty() == false)
	{
		unsigned long *relocation = relocations.front();
		relocations.pop();
		unsigned long *reloc = *(unsigned long **)relocation;
		if (reloc != 0)
		{
			unsigned long fixup = DoRelocation(reloc);
			if (fixup == 0xFFFFFFFF)
			{
				fixup = 0;
			}
			*relocation = fixup;
			unsigned long fixup2 = DoRelocation(relocation);
			fixups.push(fixup2);
		}
	}
	OvlHeader h;
	h.magic = 0x4B524746;
	h.References = 0;
	h.reserved = 0;
	h.version = 1;
	OvlHeader2 h2;
	h2.FileTypeCount = 3;
	h2.unk = 0;
	unsigned short len1 = 4;
	char *str1 = "FGDK";
	unsigned short len2 = 0xC;
	char *str2 = "FlexiTexture";
	unsigned long type = 1;
	unsigned short len3 = 3;
	char *str3 = "ftx";
	f = fopen(OutOvlFiles[0].FileName,"wb");
	fwrite(&h,sizeof(h),1,f);
	fwrite(&h2,sizeof(h2),1,f);
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "RCT3";
	len2 = 0xB;
	str2 = "SceneryItemVisual";
	type = 1;
	len3 = 3;
	str3 = "svd";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "RCT3";
	len2 = 0xB;
	str2 = "StaticShape";
	type = 1;
	len3 = 3;
	str3 = "shs";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	for (i = 0;i < 9;i++)
	{
		fwrite(&OutOvlFiles[0].Types[i].count,sizeof(OutOvlFiles[0].Types[i].count),1,f);
	}
	for (i = 0;i < 9;i++)
	{
		for (unsigned long j = 0;j < OutOvlFiles[0].Types[i].count;j++)
		{
			fwrite(&OutOvlFiles[0].Types[i].dataptr[j].size,sizeof(OutOvlFiles[0].Types[i].dataptr[j].size),1,f);
			fwrite(OutOvlFiles[0].Types[i].dataptr[j].data,OutOvlFiles[0].Types[i].dataptr[j].size,1,f);
		}
	}
	unsigned long size = fixups.size();
	fwrite(&size,sizeof(size),1,f);
	while (fixups.empty() == false)
	{
		unsigned long fixup = fixups.front();
		fixups.pop();
		fwrite(&fixup,sizeof(fixup),1,f);
	}
	fclose(f);
	sprintf(OutOvlFiles[0].FileName,"%s.common.ovl",filename);
	h.magic = 0x4B524746;
	h.References = 0;
	h.reserved = 0;
	h.version = 1;
	h2.FileTypeCount = 3;
	h2.unk = 0;
	len1 = 4;
	str1 = "FGDK";
	len2 = 0xC;
	str2 = "FlexiTexture";
	type = 1;
	len3 = 3;
	str3 = "ftx";
	f = fopen(OutOvlFiles[0].FileName,"wb");
	fwrite(&h,sizeof(h),1,f);
	fwrite(&h2,sizeof(h2),1,f);
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "RCT3";
	len2 = 0xB;
	str2 = "SceneryItemVisual";
	type = 1;
	len3 = 3;
	str3 = "svd";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "RCT3";
	len2 = 0xB;
	str2 = "StaticShape";
	type = 1;
	len3 = 3;
	str3 = "shs";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	unsigned long num = 0;
	for (i = 0;i < 9;i++)
	{
		fwrite(&num,sizeof(num),1,f);
	}
	fwrite(&num,sizeof(num),1,f);
	fclose(f);
	delete names;
	delete svd;
	delete shape;
	delete data;
	delete texturedata;
	delete Sym;
	delete Ldr;
	delete Ref;
	return true;
}
BOOL CALLBACK CreateSceneryDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	OPENFILENAME ofn;
	char sfile[MAX_PATH] = "";
	SceneryDlgHwnd = hwnd;
	switch(Message)
	{
		case WM_CLOSE:
			EndDialog(hwnd, 0);
		break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDCANCEL:
					EndDialog(hwnd, 0);
				break;
				case IDC_OPENASE:
					ZeroMemory(&ofn, sizeof(ofn));
					ofn.lStructSize = sizeof(ofn);
					ofn.hwndOwner = hwnd;
					ofn.lpstrFilter = "ASE Files (*.ase)\0*.ase\0All Files (*.*)\0*.*\0";
					ofn.lpstrFile = sfile;
					ofn.lpstrTitle = "Open ASE File";
					ofn.nMaxFile = MAX_PATH;
					ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT | OFN_NOCHANGEDIR;
					if (GetOpenFileName(&ofn))
					{
						SetDlgItemText(hwnd,IDC_ASE,sfile);
					}
				break;
				case IDC_OPENTEXTURE:
					ZeroMemory(&ofn, sizeof(ofn));
					ofn.lStructSize = sizeof(ofn);
					ofn.hwndOwner = hwnd;
					ofn.lpstrFilter = "Textures (*.bmp)\0*.bmp\0All Files (*.*)\0*.*\0";
					ofn.lpstrFile = sfile;
					ofn.lpstrTitle = "Open Texture";
					ofn.nMaxFile = MAX_PATH;
					ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT | OFN_NOCHANGEDIR;
					if (GetOpenFileName(&ofn))
					{
						SetDlgItemText(hwnd,IDC_TEXTURE,sfile);
					}
				break;
				case IDC_SAVE:
					unsigned long TextureLen = GetWindowTextLength(GetDlgItem(hwnd,IDC_TEXTURE));
					unsigned long ASELen = GetWindowTextLength(GetDlgItem(hwnd,IDC_ASE));
					unsigned long TXSLen = GetWindowTextLength(GetDlgItem(hwnd,IDC_TXS));
					unsigned long RecolorLen = GetWindowTextLength(GetDlgItem(hwnd,IDC_RECOLOR)); //added by KLN
					if ((TXSLen != 0) && (ASELen != 0) && (TextureLen != 0) && (RecolorLen != 0)) //modded by KLN
					{
						TextureLen++;
						ASELen++;
						TXSLen++;
						RecolorLen++; //added by KLN
						char *TXS = new char[TXSLen];
						char *Texture = new char[TextureLen];
						char *ASE = new char[ASELen];
						char *Recolor = new char[RecolorLen]; //added by KLN
						GetDlgItemText(hwnd,IDC_TEXTURE,Texture,TextureLen);
						GetDlgItemText(hwnd,IDC_ASE,ASE,ASELen);
						GetDlgItemText(hwnd,IDC_TXS,TXS,TXSLen);
						GetDlgItemText(hwnd,IDC_RECOLOR,Recolor,RecolorLen); //added by KLN
						ZeroMemory(&ofn, sizeof(ofn));
						ofn.lStructSize = sizeof(ofn);
						ofn.hwndOwner = hwnd;
						ofn.lpstrFilter = "Overlay Files (*.common.ovl)\0*.common.ovl\0";
						ofn.lpstrFile = sfile;
						ofn.lpstrTitle = "Save OVL file";
						ofn.lpstrDefExt = ".common.ovl";
						ofn.nMaxFile = MAX_PATH;
						ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT;
						if (GetSaveFileName(&ofn))
						{
							strchr(sfile,'.')[0] = 0;
							char name[MAX_PATH];
							strcpy(name,(strrchr(sfile,'\\')+1));
							SaveOvl(ASE,Texture,name,sfile,TXS,Recolor); //modded by KLN
							EndDialog(hwnd, 1);
						}
						delete TXS;
						delete Texture;
						delete ASE;
						delete Recolor; //added by KLN
					}
					else
					{
						MessageBox(hwnd,"Nothing to save","Error",MB_OK);
					}
				break;
			}
			break;
		default:
			return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK TextEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_INITDIALOG:
		if (CurrentTextString != -1)
		{
			SetDlgItemText(hwnd,IDC_NAME,TextStrings[CurrentTextString]->name);
			SetDlgItemText(hwnd,IDC_TEXT,TextStrings[CurrentTextString]->text);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDOK:
			unsigned long len,len2;
			len = GetWindowTextLength(GetDlgItem(hwnd,IDC_NAME));
			len2 = GetWindowTextLength(GetDlgItem(hwnd,IDC_TEXT));
			if ((len != 0) && (len2 != 0))
			{
				save = true;
				if (CurrentTextString != -1)
				{
					delete TextStrings[CurrentTextString]->name;
					delete TextStrings[CurrentTextString]->text;
				}
				else
				{
					Text *t = new Text;
					TextStrings.push_back(t);
					CurrentTextString = TextStrings.size()-1;
				}
				TextStrings[CurrentTextString]->name = new char[len+1];
				GetDlgItemText(hwnd,IDC_NAME,TextStrings[CurrentTextString]->name,len+1);
				TextStrings[CurrentTextString]->text = new char[len2+1];
				GetDlgItemText(hwnd,IDC_TEXT,TextStrings[CurrentTextString]->text,len2+1);
			}
			EndDialog(hwnd, 1);
			break;
		case IDCANCEL:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
bool enabled;
BOOL CALLBACK GSIEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_INITDIALOG:
		SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
		for (IconTexturesIterator = IconTextures.begin();IconTexturesIterator != IconTextures.end();IconTexturesIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconTexturesIterator)->name);
		}
		if (CurrentIcon != -1)
		{
			SetDlgItemText(hwnd,IDC_NAME,Icons[CurrentIcon]->name);
			SetDlgItemInt(hwnd,IDC_TOP,Icons[CurrentIcon]->top,FALSE);
			SetDlgItemInt(hwnd,IDC_LEFT,Icons[CurrentIcon]->left,FALSE);
			SetDlgItemInt(hwnd,IDC_BOTTOM,Icons[CurrentIcon]->bottom,FALSE);
			SetDlgItemInt(hwnd,IDC_RIGHT,Icons[CurrentIcon]->right,FALSE);
			unsigned long pos = SendDlgItemMessage(hwnd,IDC_LIST,LB_FINDSTRING,-1,(WPARAM)Icons[CurrentIcon]->texture);
			SendDlgItemMessage(hwnd,IDC_LIST,LB_SETCURSEL,pos,0);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDOK:
			unsigned long len;
			len = GetWindowTextLength(GetDlgItem(hwnd,IDC_NAME));
			BOOL fail1,fail2,fail3,fail4;
			unsigned long top,left,bottom,right,texture;
			top = GetDlgItemInt(hwnd,IDC_TOP,&fail1,FALSE);
			left = GetDlgItemInt(hwnd,IDC_LEFT,&fail2,FALSE);
			bottom = GetDlgItemInt(hwnd,IDC_BOTTOM,&fail3,FALSE);
			right = GetDlgItemInt(hwnd,IDC_RIGHT,&fail4,FALSE);
			texture = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
			if ((len != 0) && (fail1 == TRUE) && (fail2 == TRUE) && (fail3 == TRUE) && (fail4 == TRUE) && (texture != -1))
			{
				save = true;
				if (CurrentIcon != -1)
				{
					delete Icons[CurrentIcon]->name;
					delete Icons[CurrentIcon]->texture;
				}
				else
				{
					Icon *t = new Icon;
					Icons.push_back(t);
					CurrentIcon = Icons.size()-1;
				}
				Icons[CurrentIcon]->name = new char[len+1];
				GetDlgItemText(hwnd,IDC_NAME,Icons[CurrentIcon]->name,len+1);
				Icons[CurrentIcon]->top = top;
				Icons[CurrentIcon]->left = left;
				Icons[CurrentIcon]->bottom = bottom;
				Icons[CurrentIcon]->right = right;
				Icons[CurrentIcon]->texture = new char[strlen(IconTextures[texture]->name)+1];
				strcpy(Icons[CurrentIcon]->texture,IconTextures[texture]->name);
			}
			EndDialog(hwnd, 1);
			break;
		case IDCANCEL:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK TextureEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	char sfile[MAX_PATH] = "";
	switch(Message)
	{
	case WM_INITDIALOG:
		if (CurrentIconTexture != -1)
		{
			SetDlgItemText(hwnd,IDC_NAME,IconTextures[CurrentIconTexture]->name);
			SetDlgItemText(hwnd,IDC_FILENAME,IconTextures[CurrentIconTexture]->filename);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_LOAD:
			OPENFILENAME ofn;
			ZeroMemory(&ofn, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = hwnd;
			ofn.lpstrFilter = "Textures (*.bmp)\0*.bmp\0All Files (*.*)\0*.*\0";
			ofn.lpstrFile = sfile;
			ofn.lpstrTitle = "Open Icon Texture";
			ofn.nMaxFile = MAX_PATH;
			ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT | OFN_NOCHANGEDIR;
			if (GetOpenFileName(&ofn))
			{
				SetDlgItemText(hwnd,IDC_FILENAME,sfile);
			}
			break;
		case IDOK:
			unsigned long len,len2;
			len = GetWindowTextLength(GetDlgItem(hwnd,IDC_NAME));
			len2 = GetWindowTextLength(GetDlgItem(hwnd,IDC_FILENAME));
			if ((len != 0) && (len2 != 0))
			{
				save = true;
				if (CurrentIconTexture != -1)
				{
					delete IconTextures[CurrentIconTexture]->name;
					delete IconTextures[CurrentIconTexture]->filename;
				}
				else
				{
					IconTexture *t = new IconTexture;
					IconTextures.push_back(t);
					CurrentIconTexture = IconTextures.size()-1;
				}
				IconTextures[CurrentIconTexture]->name = new char[len+1];
				GetDlgItemText(hwnd,IDC_NAME,IconTextures[CurrentIconTexture]->name,len+1);
				IconTextures[CurrentIconTexture]->filename = new char[len2+1];
				GetDlgItemText(hwnd,IDC_FILENAME,IconTextures[CurrentIconTexture]->filename,len2+1);
			}
			EndDialog(hwnd, 1);
			break;
		case IDCANCEL:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK SIDEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	char sfile[MAX_PATH] = "";
	switch(Message)
	{
	case WM_INITDIALOG:
		SendDlgItemMessage(hwnd,IDC_SCENICON,CB_RESETCONTENT,0,0);
		for (IconsIterator = Icons.begin();IconsIterator != Icons.end();IconsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_SCENICON,CB_ADDSTRING,0,(LPARAM)(*IconsIterator)->name);
		}
		SendDlgItemMessage(hwnd,IDC_WALLICON,CB_RESETCONTENT,0,0);
		for (IconsIterator = Icons.begin();IconsIterator != Icons.end();IconsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_WALLICON,CB_ADDSTRING,0,(LPARAM)(*IconsIterator)->name);
		}
		SendDlgItemMessage(hwnd,IDC_WALLNAME,CB_RESETCONTENT,0,0);
		for (TextStringsIterator = TextStrings.begin();TextStringsIterator != TextStrings.end();TextStringsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_WALLNAME,CB_ADDSTRING,0,(LPARAM)(*TextStringsIterator)->name);
		}
		SendDlgItemMessage(hwnd,IDC_SCENNAME,CB_RESETCONTENT,0,0);
		for (TextStringsIterator = TextStrings.begin();TextStringsIterator != TextStrings.end();TextStringsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_SCENNAME,CB_ADDSTRING,0,(LPARAM)(*TextStringsIterator)->name);
		}
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Tree");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Plant");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Shrub");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Flowers");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Fence");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Wall Misc");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Path Lamp");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Small Scenery");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Medium Scenery");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Large Scenery");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Anamatronic Scenery");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Misc Scenery");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Support Middle");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Support Top");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Support Bottom");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Support Bottom Extra");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Support Girder");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Support Cap");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Ride Track");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Path");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Park Entrance");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Litter");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Guest Injection");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Ride");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Ride Entrance");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Ride Exit");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Keep Clear Fence");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Stall");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Ride Event");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Firework");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Litter Bin");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Bench");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Sign");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Photo Point");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Wall Straight");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Wall Roof");
		SendDlgItemMessage(hwnd,IDC_TYPE,CB_ADDSTRING,0,(WPARAM)"Wall Corner");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"None");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Adventure");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Crazy Golf");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Crazy Golf Holes");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Girder");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Ornate");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Asphalt");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Crazy");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Dirt");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Leafy");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Marble");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Tarmac");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Queue Set 1");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Roman Pillar");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Scaffolding");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Space");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Spooky");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Steel");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"TS1");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"TS2");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"TS3");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"TS4");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"TS5");
		SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_ADDSTRING,0,(WPARAM)"Wild West");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 1");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 2");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 3");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 4");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 5");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 6");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 7");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 8");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 9");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 10");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 11");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 12");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 13");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 14");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 15");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 16");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 17");
		SendDlgItemMessage(hwnd,IDC_UNK4,CB_ADDSTRING,0,(WPARAM)"unknown 18");
		SendDlgItemMessage(hwnd,IDC_UNK17,CB_ADDSTRING,0,(WPARAM)"unknown 1");
		SendDlgItemMessage(hwnd,IDC_UNK17,CB_ADDSTRING,0,(WPARAM)"unknown 2");
		SendDlgItemMessage(hwnd,IDC_UNK17,CB_ADDSTRING,0,(WPARAM)"unknown 3");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Full Tile");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Path Edge (Inner)");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Path Edge (Outer)");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Wall");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"1/4 Tile");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"1/2 Tile");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Path Center");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Corner");
		SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_ADDSTRING,0,(WPARAM)"Path Edge (Join)");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"None");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Guest Injection Point");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Litter Bin");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Bench");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Park Entrance");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Queue Line Entertainer");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Ride Event");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Sign");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Visa Point");
		SendDlgItemMessage(hwnd,IDC_PARAMS,CB_ADDSTRING,0,(WPARAM)"Waterable");
		if (CurrentScenery != -1)
		{
			SetDlgItemText(hwnd,IDC_OVL,SceneryItems[CurrentScenery]->ovl);
			SetDlgItemText(hwnd,IDC_LOCATION,SceneryItems[CurrentScenery]->location);
			SetDlgItemInt(hwnd,IDC_COST,SceneryItems[CurrentScenery]->cost,TRUE);
			SetDlgItemInt(hwnd,IDC_REFUND,SceneryItems[CurrentScenery]->refund,TRUE);
			SetDlgItemInt(hwnd,IDC_XSQUARES,SceneryItems[CurrentScenery]->sizex,FALSE);
			SetDlgItemInt(hwnd,IDC_YSQUARES,SceneryItems[CurrentScenery]->sizey,FALSE);
			SetDlgItemInt(hwnd,IDC_UNK31,SceneryItems[CurrentScenery]->unk31,FALSE);
			SetDlgItemInt(hwnd,IDC_UNK32,SceneryItems[CurrentScenery]->unk32,FALSE);
			SetDlgItemInt(hwnd,IDC_UNK33,SceneryItems[CurrentScenery]->unk33,FALSE);
			CheckDlgButton(hwnd,IDC_FLAG33,SceneryItems[CurrentScenery]->flag33);
			CheckDlgButton(hwnd,IDC_FLAG34,SceneryItems[CurrentScenery]->flag34);
			CheckDlgButton(hwnd,IDC_FLAG35,SceneryItems[CurrentScenery]->flag35);
			CheckDlgButton(hwnd,IDC_FLAG36,SceneryItems[CurrentScenery]->flag36);
			CheckDlgButton(hwnd,IDC_FLAG37,SceneryItems[CurrentScenery]->flag37);
			CheckDlgButton(hwnd,IDC_FLAG38,SceneryItems[CurrentScenery]->flag38);
			CheckDlgButton(hwnd,IDC_FLAG39,SceneryItems[CurrentScenery]->flag39);
			CheckDlgButton(hwnd,IDC_FLAG40,SceneryItems[CurrentScenery]->flag40);
			CheckDlgButton(hwnd,IDC_FLAG41,SceneryItems[CurrentScenery]->flag41);
			CheckDlgButton(hwnd,IDC_FLAG42,SceneryItems[CurrentScenery]->flag42);
			CheckDlgButton(hwnd,IDC_FLAG43,SceneryItems[CurrentScenery]->flag43);
			CheckDlgButton(hwnd,IDC_FLAG44,SceneryItems[CurrentScenery]->flag44);
			CheckDlgButton(hwnd,IDC_FLAG45,SceneryItems[CurrentScenery]->flag45);
			CheckDlgButton(hwnd,IDC_FLAG46,SceneryItems[CurrentScenery]->flag46);
			CheckDlgButton(hwnd,IDC_FLAG47,SceneryItems[CurrentScenery]->flag47);
			CheckDlgButton(hwnd,IDC_FLAG48,SceneryItems[CurrentScenery]->flag48);
			CheckDlgButton(hwnd,IDC_FLAG49,SceneryItems[CurrentScenery]->flag49);
			CheckDlgButton(hwnd,IDC_FLAG50,SceneryItems[CurrentScenery]->flag50);
			CheckDlgButton(hwnd,IDC_FLAG51,SceneryItems[CurrentScenery]->flag51);
			CheckDlgButton(hwnd,IDC_FLAG52,SceneryItems[CurrentScenery]->flag52);
			CheckDlgButton(hwnd,IDC_FLAG53,SceneryItems[CurrentScenery]->flag53);
			CheckDlgButton(hwnd,IDC_FLAG54,SceneryItems[CurrentScenery]->flag54);
			CheckDlgButton(hwnd,IDC_FLAG55,SceneryItems[CurrentScenery]->flag55);
			CheckDlgButton(hwnd,IDC_FLAG56,SceneryItems[CurrentScenery]->flag56);
			CheckDlgButton(hwnd,IDC_FLAG57,SceneryItems[CurrentScenery]->flag57);
			CheckDlgButton(hwnd,IDC_FLAG58,SceneryItems[CurrentScenery]->flag58);
			CheckDlgButton(hwnd,IDC_FLAG59,SceneryItems[CurrentScenery]->flag59);
			CheckDlgButton(hwnd,IDC_FLAG60,SceneryItems[CurrentScenery]->flag60);
			CheckDlgButton(hwnd,IDC_FLAG61,SceneryItems[CurrentScenery]->flag61);
			CheckDlgButton(hwnd,IDC_FLAG62,SceneryItems[CurrentScenery]->flag62);
			CheckDlgButton(hwnd,IDC_FLAG63,SceneryItems[CurrentScenery]->flag63);
			CheckDlgButton(hwnd,IDC_FLAG64,SceneryItems[CurrentScenery]->flag64);
			SetDlgItemInt(hwnd,IDC_DUNK2,SceneryItems[CurrentScenery]->dunk2,TRUE);
			SetDlgItemInt(hwnd,IDC_DUNK3,SceneryItems[CurrentScenery]->dunk3,FALSE);
			SetDlgItemInt(hwnd,IDC_DUNK4,SceneryItems[CurrentScenery]->dunk4,TRUE);
			SetDlgItemInt(hwnd,IDC_DUNK5,SceneryItems[CurrentScenery]->dunk5,FALSE);
			CheckDlgButton(hwnd,IDC_FLAG1,SceneryItems[CurrentScenery]->flag1);
			CheckDlgButton(hwnd,IDC_FLAG2,SceneryItems[CurrentScenery]->flag2);
			CheckDlgButton(hwnd,IDC_FLAG3,SceneryItems[CurrentScenery]->flag3);
			CheckDlgButton(hwnd,IDC_FLAG4,SceneryItems[CurrentScenery]->flag4);
			CheckDlgButton(hwnd,IDC_FLAG5,SceneryItems[CurrentScenery]->flag5);
			CheckDlgButton(hwnd,IDC_FLAG6,SceneryItems[CurrentScenery]->flag6);
			CheckDlgButton(hwnd,IDC_FLAG7,SceneryItems[CurrentScenery]->flag7);
			CheckDlgButton(hwnd,IDC_FLAG8,SceneryItems[CurrentScenery]->flag8);
			CheckDlgButton(hwnd,IDC_FLAG9,SceneryItems[CurrentScenery]->flag9);
			CheckDlgButton(hwnd,IDC_FLAG10,SceneryItems[CurrentScenery]->flag10);
			CheckDlgButton(hwnd,IDC_FLAG11,SceneryItems[CurrentScenery]->flag11);
			CheckDlgButton(hwnd,IDC_FLAG12,SceneryItems[CurrentScenery]->flag12);
			CheckDlgButton(hwnd,IDC_FLAG13,SceneryItems[CurrentScenery]->flag13);
			CheckDlgButton(hwnd,IDC_FLAG14,SceneryItems[CurrentScenery]->flag14);
			CheckDlgButton(hwnd,IDC_FLAG15,SceneryItems[CurrentScenery]->flag15);
			CheckDlgButton(hwnd,IDC_FLAG16,SceneryItems[CurrentScenery]->flag16);
			CheckDlgButton(hwnd,IDC_FLAG17,SceneryItems[CurrentScenery]->flag17);
			CheckDlgButton(hwnd,IDC_FLAG18,SceneryItems[CurrentScenery]->flag18);
			CheckDlgButton(hwnd,IDC_FLAG19,SceneryItems[CurrentScenery]->flag19);
			CheckDlgButton(hwnd,IDC_FLAG20,SceneryItems[CurrentScenery]->flag20);
			CheckDlgButton(hwnd,IDC_FLAG21,SceneryItems[CurrentScenery]->flag21);
			CheckDlgButton(hwnd,IDC_FLAG22,SceneryItems[CurrentScenery]->flag22);
			CheckDlgButton(hwnd,IDC_FLAG23,SceneryItems[CurrentScenery]->flag23);
			CheckDlgButton(hwnd,IDC_FLAG24,SceneryItems[CurrentScenery]->flag24);
			CheckDlgButton(hwnd,IDC_FLAG25,SceneryItems[CurrentScenery]->flag25);
			CheckDlgButton(hwnd,IDC_FLAG26,SceneryItems[CurrentScenery]->flag26);
			CheckDlgButton(hwnd,IDC_FLAG27,SceneryItems[CurrentScenery]->flag27);
			CheckDlgButton(hwnd,IDC_FLAG28,SceneryItems[CurrentScenery]->flag28);
			CheckDlgButton(hwnd,IDC_FLAG29,SceneryItems[CurrentScenery]->flag29);
			CheckDlgButton(hwnd,IDC_FLAG30,SceneryItems[CurrentScenery]->flag30);
			CheckDlgButton(hwnd,IDC_FLAG31,SceneryItems[CurrentScenery]->flag31);
			CheckDlgButton(hwnd,IDC_FLAG32,SceneryItems[CurrentScenery]->flag32);
			SendDlgItemMessage(hwnd,IDC_TYPE,CB_SETCURSEL,SceneryItems[CurrentScenery]->type,0);
			SendDlgItemMessage(hwnd,IDC_UNK4,CB_SETCURSEL,SceneryItems[CurrentScenery]->unk4,0);
			SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_SETCURSEL,SceneryItems[CurrentScenery]->supportstype,0);
			SendDlgItemMessage(hwnd,IDC_UNK17,CB_SETCURSEL,SceneryItems[CurrentScenery]->unk17,0);
			SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_SETCURSEL,SceneryItems[CurrentScenery]->sizeflag,0);
			SendDlgItemMessage(hwnd,IDC_PARAMS,CB_SETCURSEL,SceneryItems[CurrentScenery]->extra,0);
			unsigned long pos = SendDlgItemMessage(hwnd,IDC_SCENICON,CB_FINDSTRING,-1,(WPARAM)SceneryItems[CurrentScenery]->icon);
			SendDlgItemMessage(hwnd,IDC_SCENICON,CB_SETCURSEL,pos,0);
			pos = SendDlgItemMessage(hwnd,IDC_SCENNAME,CB_FINDSTRING,-1,(WPARAM)SceneryItems[CurrentScenery]->name);
			SendDlgItemMessage(hwnd,IDC_SCENNAME,CB_SETCURSEL,pos,0);
			if (SceneryItems[CurrentScenery]->wallicon != 0)
			{
				enabled = true;
				pos = SendDlgItemMessage(hwnd,IDC_WALLNAME,CB_FINDSTRING,-1,(WPARAM)SceneryItems[CurrentScenery]->wallname);
				SendDlgItemMessage(hwnd,IDC_WALLNAME,CB_SETCURSEL,pos,0);
				pos = SendDlgItemMessage(hwnd,IDC_WALLICON,CB_FINDSTRING,-1,(WPARAM)SceneryItems[CurrentScenery]->wallicon);
				SendDlgItemMessage(hwnd,IDC_WALLICON,CB_SETCURSEL,pos,0);
			}
			else
			{
				enabled = false;
				EnableWindow(GetDlgItem(hwnd,IDC_WALLICON),false);
				EnableWindow(GetDlgItem(hwnd,IDC_WALLNAME),false);
			}
			if (enabled == true)
			{
				CheckDlgButton(hwnd,IDC_DISABLE,false);
			}
			else
			{
				CheckDlgButton(hwnd,IDC_DISABLE,true);
			}
			char temp[100];
			sprintf(temp,"%f",SceneryItems[CurrentScenery]->position.x);
			SetDlgItemText(hwnd,IDC_XPOS,temp);
			sprintf(temp,"%f",SceneryItems[CurrentScenery]->position.y);
			SetDlgItemText(hwnd,IDC_YPOS,temp);
			sprintf(temp,"%f",SceneryItems[CurrentScenery]->position.z);
			SetDlgItemText(hwnd,IDC_ZPOS,temp);
			sprintf(temp,"%f",SceneryItems[CurrentScenery]->size.x);
			SetDlgItemText(hwnd,IDC_XSIZE,temp);
			sprintf(temp,"%f",SceneryItems[CurrentScenery]->size.y);
			SetDlgItemText(hwnd,IDC_YSIZE,temp);
			sprintf(temp,"%f",SceneryItems[CurrentScenery]->size.z);
			SetDlgItemText(hwnd,IDC_ZSIZE,temp);
		}
		else
		{
			enabled = true;
			CheckDlgButton(hwnd,IDC_DISABLE,false);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_DISABLE:
			if (enabled == true)
			{
				enabled = false;
				EnableWindow(GetDlgItem(hwnd,IDC_WALLICON),false);
				EnableWindow(GetDlgItem(hwnd,IDC_WALLNAME),false);
			}
			else
			{
				enabled = true;
				EnableWindow(GetDlgItem(hwnd,IDC_WALLICON),true);
				EnableWindow(GetDlgItem(hwnd,IDC_WALLNAME),true);
			}
			break;
		case IDOK:
			unsigned long ovllen, loclen, sizex, sizey, wallicon, icon, wallname, name, type, unk4, supportstype, unk17;
			long cost, refund;
			unsigned long extra, sizeflag;
			unsigned long posxlen, posylen, poszlen, sizexlen, sizeylen, sizezlen;
			unsigned long unk31,unk32,unk33,dunk3,dunk5;
			long dunk2,dunk4;
			BOOL unk31fail,unk32fail,unk33fail,dunk2fail,dunk3fail,dunk4fail,dunk5fail;
			char *temp;
			BOOL costfail, refundfail, sizexfail, sizeyfail;
			ovllen = GetWindowTextLength(GetDlgItem(hwnd,IDC_OVL));
			loclen = GetWindowTextLength(GetDlgItem(hwnd,IDC_LOCATION));
			posxlen = GetWindowTextLength(GetDlgItem(hwnd,IDC_XPOS));
			posylen = GetWindowTextLength(GetDlgItem(hwnd,IDC_YPOS));
			poszlen = GetWindowTextLength(GetDlgItem(hwnd,IDC_ZPOS));
			sizexlen = GetWindowTextLength(GetDlgItem(hwnd,IDC_XSIZE));
			sizeylen = GetWindowTextLength(GetDlgItem(hwnd,IDC_YSIZE));
			sizezlen = GetWindowTextLength(GetDlgItem(hwnd,IDC_ZSIZE));
			sizex = GetDlgItemInt(hwnd,IDC_XSQUARES,&sizexfail,FALSE);
			sizey = GetDlgItemInt(hwnd,IDC_YSQUARES,&sizeyfail,FALSE);
			refund = GetDlgItemInt(hwnd,IDC_REFUND,&refundfail,TRUE);
			cost = GetDlgItemInt(hwnd,IDC_COST,&costfail,TRUE);
			unk31 = GetDlgItemInt(hwnd,IDC_UNK31,&unk31fail,FALSE);
			unk32 = GetDlgItemInt(hwnd,IDC_UNK32,&unk32fail,FALSE);
			unk33 = GetDlgItemInt(hwnd,IDC_UNK33,&unk33fail,FALSE);
			dunk2 = GetDlgItemInt(hwnd,IDC_DUNK2,&dunk2fail,TRUE);
			dunk3 = GetDlgItemInt(hwnd,IDC_DUNK3,&dunk3fail,FALSE);
			dunk4 = GetDlgItemInt(hwnd,IDC_DUNK4,&dunk4fail,TRUE);
			dunk5 = GetDlgItemInt(hwnd,IDC_DUNK5,&dunk5fail,FALSE);
			if (enabled == true)
			{
				wallicon = SendDlgItemMessage(hwnd,IDC_WALLICON,CB_GETCURSEL,0,0);
				wallname = SendDlgItemMessage(hwnd,IDC_WALLNAME,CB_GETCURSEL,0,0);
			}
			icon = SendDlgItemMessage(hwnd,IDC_SCENICON,CB_GETCURSEL,0,0);
			name = SendDlgItemMessage(hwnd,IDC_SCENNAME,CB_GETCURSEL,0,0);
			type = SendDlgItemMessage(hwnd,IDC_TYPE,CB_GETCURSEL,0,0);
			unk4 = SendDlgItemMessage(hwnd,IDC_UNK4,CB_GETCURSEL,0,0);
			supportstype = SendDlgItemMessage(hwnd,IDC_SUPPORTS,CB_GETCURSEL,0,0);
			unk17 = SendDlgItemMessage(hwnd,IDC_UNK17,CB_GETCURSEL,0,0);
			sizeflag = SendDlgItemMessage(hwnd,IDC_SIZEFLAG,CB_GETCURSEL,0,0);
			extra = SendDlgItemMessage(hwnd,IDC_PARAMS,CB_GETCURSEL,0,0);
			if ((ovllen != 0) && (loclen != 0) && (posxlen != 0) && (posylen != 0) && (poszlen != 0) && (sizexlen != 0) && (sizeylen != 0) && (sizezlen != 0) && (unk31fail == TRUE) && (unk32fail == TRUE) && (unk33fail == TRUE) && (dunk2fail == TRUE) && (dunk3fail == TRUE) && (dunk4fail == TRUE) && (dunk5fail == TRUE) && (sizexfail == TRUE) && (sizeyfail == TRUE) && (refundfail == TRUE) && (costfail == TRUE) && (icon != -1) && (name != -1) && (type != -1) && (unk4 != -1) && (supportstype != -1) && (unk17 != -1) && (sizeflag != -1) && (extra != -1))
			{
				if ((enabled == false) || ((enabled == true) && (wallicon != -1) && (wallname != -1)))
				{
					save = true;
					if (CurrentScenery != -1)
					{
						delete SceneryItems[CurrentScenery]->name;
						delete SceneryItems[CurrentScenery]->ovl;
						delete SceneryItems[CurrentScenery]->location;
						delete SceneryItems[CurrentScenery]->icon;
						if (SceneryItems[CurrentScenery]->wallicon != 0)
						{
							delete SceneryItems[CurrentScenery]->wallicon;
							delete SceneryItems[CurrentScenery]->wallname;
						}
					}
					else
					{
						Scenery *t = new Scenery;
						SceneryItems.push_back(t);
						CurrentScenery = SceneryItems.size()-1;
					}
					Scenery *s = SceneryItems[CurrentScenery];
					s->ovl = new char[ovllen+1];
					GetDlgItemText(hwnd,IDC_OVL,s->ovl,ovllen+1);
					s->location = new char[loclen+1];
					GetDlgItemText(hwnd,IDC_LOCATION,s->location,loclen+1);
					temp = new char[posxlen+1];
					GetDlgItemText(hwnd,IDC_XPOS,temp,posxlen+1);
					s->position.x = (float)atof(temp);
					delete temp;
					temp = new char[posylen+1];
					GetDlgItemText(hwnd,IDC_YPOS,temp,posylen+1);
					s->position.y = (float)atof(temp);
					delete temp;
					temp = new char[poszlen+1];
					GetDlgItemText(hwnd,IDC_XPOS,temp,poszlen+1);
					s->position.z = (float)atof(temp);
					delete temp;
					temp = new char[sizexlen+1];
					GetDlgItemText(hwnd,IDC_XSIZE,temp,sizexlen+1);
					s->size.x = (float)atof(temp);
					delete temp;
					temp = new char[sizeylen+1];
					GetDlgItemText(hwnd,IDC_YSIZE,temp,sizeylen+1);
					s->size.y = (float)atof(temp);
					delete temp;
					temp = new char[sizezlen+1];
					GetDlgItemText(hwnd,IDC_ZSIZE,temp,sizezlen+1);
					s->size.z = (float)atof(temp);
					delete temp;
					s->cost = cost;
					s->refund = refund;
					s->sizex = sizex;
					s->sizey = sizey;
					s->unk31 = unk31;
					s->unk32 = unk32;
					s->unk33 = unk33;
					s->dunk2 = dunk2;
					s->dunk3 = dunk3;
					s->dunk4 = dunk4;
					s->dunk5 = dunk5;
					s->flag1 = IsDlgButtonChecked(hwnd,IDC_FLAG1);
					s->flag2 = IsDlgButtonChecked(hwnd,IDC_FLAG2);
					s->flag3 = IsDlgButtonChecked(hwnd,IDC_FLAG3);
					s->flag4 = IsDlgButtonChecked(hwnd,IDC_FLAG4);
					s->flag5 = IsDlgButtonChecked(hwnd,IDC_FLAG5);
					s->flag6 = IsDlgButtonChecked(hwnd,IDC_FLAG6);
					s->flag7 = IsDlgButtonChecked(hwnd,IDC_FLAG7);
					s->flag8 = IsDlgButtonChecked(hwnd,IDC_FLAG8);
					s->flag9 = IsDlgButtonChecked(hwnd,IDC_FLAG9);
					s->flag10 = IsDlgButtonChecked(hwnd,IDC_FLAG10);
					s->flag11 = IsDlgButtonChecked(hwnd,IDC_FLAG11);
					s->flag12 = IsDlgButtonChecked(hwnd,IDC_FLAG12);
					s->flag13 = IsDlgButtonChecked(hwnd,IDC_FLAG13);
					s->flag14 = IsDlgButtonChecked(hwnd,IDC_FLAG14);
					s->flag15 = IsDlgButtonChecked(hwnd,IDC_FLAG15);
					s->flag16 = IsDlgButtonChecked(hwnd,IDC_FLAG16);
					s->flag17 = IsDlgButtonChecked(hwnd,IDC_FLAG17);
					s->flag18 = IsDlgButtonChecked(hwnd,IDC_FLAG18);
					s->flag19 = IsDlgButtonChecked(hwnd,IDC_FLAG19);
					s->flag20 = IsDlgButtonChecked(hwnd,IDC_FLAG20);
					s->flag21 = IsDlgButtonChecked(hwnd,IDC_FLAG21);
					s->flag22 = IsDlgButtonChecked(hwnd,IDC_FLAG22);
					s->flag23 = IsDlgButtonChecked(hwnd,IDC_FLAG23);
					s->flag24 = IsDlgButtonChecked(hwnd,IDC_FLAG24);
					s->flag25 = IsDlgButtonChecked(hwnd,IDC_FLAG25);
					s->flag26 = IsDlgButtonChecked(hwnd,IDC_FLAG26);
					s->flag27 = IsDlgButtonChecked(hwnd,IDC_FLAG27);
					s->flag28 = IsDlgButtonChecked(hwnd,IDC_FLAG28);
					s->flag29 = IsDlgButtonChecked(hwnd,IDC_FLAG29);
					s->flag30 = IsDlgButtonChecked(hwnd,IDC_FLAG30);
					s->flag31 = IsDlgButtonChecked(hwnd,IDC_FLAG31);
					s->flag32 = IsDlgButtonChecked(hwnd,IDC_FLAG32);
					s->flag33 = IsDlgButtonChecked(hwnd,IDC_FLAG33);
					s->flag34 = IsDlgButtonChecked(hwnd,IDC_FLAG34);
					s->flag35 = IsDlgButtonChecked(hwnd,IDC_FLAG35);
					s->flag36 = IsDlgButtonChecked(hwnd,IDC_FLAG36);
					s->flag37 = IsDlgButtonChecked(hwnd,IDC_FLAG37);
					s->flag38 = IsDlgButtonChecked(hwnd,IDC_FLAG38);
					s->flag39 = IsDlgButtonChecked(hwnd,IDC_FLAG39);
					s->flag40 = IsDlgButtonChecked(hwnd,IDC_FLAG40);
					s->flag41 = IsDlgButtonChecked(hwnd,IDC_FLAG41);
					s->flag42 = IsDlgButtonChecked(hwnd,IDC_FLAG42);
					s->flag43 = IsDlgButtonChecked(hwnd,IDC_FLAG43);
					s->flag44 = IsDlgButtonChecked(hwnd,IDC_FLAG44);
					s->flag45 = IsDlgButtonChecked(hwnd,IDC_FLAG45);
					s->flag46 = IsDlgButtonChecked(hwnd,IDC_FLAG46);
					s->flag47 = IsDlgButtonChecked(hwnd,IDC_FLAG47);
					s->flag48 = IsDlgButtonChecked(hwnd,IDC_FLAG48);
					s->flag49 = IsDlgButtonChecked(hwnd,IDC_FLAG49);
					s->flag50 = IsDlgButtonChecked(hwnd,IDC_FLAG50);
					s->flag51 = IsDlgButtonChecked(hwnd,IDC_FLAG51);
					s->flag52 = IsDlgButtonChecked(hwnd,IDC_FLAG52);
					s->flag53 = IsDlgButtonChecked(hwnd,IDC_FLAG53);
					s->flag54 = IsDlgButtonChecked(hwnd,IDC_FLAG54);
					s->flag55 = IsDlgButtonChecked(hwnd,IDC_FLAG55);
					s->flag56 = IsDlgButtonChecked(hwnd,IDC_FLAG56);
					s->flag57 = IsDlgButtonChecked(hwnd,IDC_FLAG57);
					s->flag58 = IsDlgButtonChecked(hwnd,IDC_FLAG58);
					s->flag59 = IsDlgButtonChecked(hwnd,IDC_FLAG59);
					s->flag60 = IsDlgButtonChecked(hwnd,IDC_FLAG60);
					s->flag61 = IsDlgButtonChecked(hwnd,IDC_FLAG61);
					s->flag62 = IsDlgButtonChecked(hwnd,IDC_FLAG62);
					s->flag63 = IsDlgButtonChecked(hwnd,IDC_FLAG63);
					s->flag64 = IsDlgButtonChecked(hwnd,IDC_FLAG64);
					s->type = type;
					s->unk4 = unk4;
					s->supportstype = supportstype;
					s->unk17 = unk17;
					s->sizeflag = sizeflag;
					s->extra = extra;
					s->icon = new char[strlen(Icons[icon]->name)+1];
					strcpy(s->icon,Icons[icon]->name);
					s->name = new char[strlen(TextStrings[name]->name)+1];
					strcpy(s->name,TextStrings[name]->name);
					if (enabled == true)
					{
						s->wallicon = new char[strlen(Icons[wallicon]->name)+1];
						strcpy(s->wallicon,Icons[wallicon]->name);
						s->wallname = new char[strlen(TextStrings[wallname]->name)+1];
						strcpy(s->wallname,TextStrings[wallname]->name);
					}
					else
					{
						s->wallicon = 0;
						s->wallname = 0;
					}
					EndDialog(hwnd, 1);
				}
			}
			break;
		case IDCANCEL:
			EndDialog(hwnd, 0);
			break;
		case IDC_SELECT:
			OPENFILENAME ofn;
			ZeroMemory(&ofn, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = hwnd;
			ofn.lpstrFilter = "Overlay Files (*.common.ovl)\0*.common.ovl\0";
			ofn.lpstrFile = sfile;
			ofn.lpstrTitle = "Open OVL File";
			ofn.nMaxFile = MAX_PATH;
			ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT | OFN_NOCHANGEDIR;
			if (GetOpenFileName(&ofn))
			{
				SetDlgItemText(hwnd,IDC_OVL,sfile);
			}
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK SIDManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_INITDIALOG:
		SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
		for (SceneryItemsIterator = SceneryItems.begin();SceneryItemsIterator != SceneryItems.end();SceneryItemsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*SceneryItemsIterator)->name);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_ADD:
			CurrentScenery = -1;
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_SIDEDIT), hwnd, SIDEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (SceneryItemsIterator = SceneryItems.begin();SceneryItemsIterator != SceneryItems.end();SceneryItemsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*SceneryItemsIterator)->name);
				}
			}
			break;
		case IDC_EDIT:
			CurrentScenery = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_SIDEDIT), hwnd, SIDEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (SceneryItemsIterator = SceneryItems.begin();SceneryItemsIterator != SceneryItems.end();SceneryItemsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*SceneryItemsIterator)->name);
				}
			}
			break;
		case IDC_DEL:
			if (MessageBox(hwnd,"Are you sure you want to delete?","Are You Sure",MB_YESNO|MB_ICONQUESTION) == IDYES)
			{
				save = true;
				CurrentScenery = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
				delete SceneryItems[CurrentScenery]->name;
				delete SceneryItems[CurrentScenery]->ovl;
				delete SceneryItems[CurrentScenery]->location;
				delete SceneryItems[CurrentScenery]->icon;
				if (SceneryItems[CurrentScenery]->wallicon != 0)
				{
					delete SceneryItems[CurrentScenery]->wallicon;
					delete SceneryItems[CurrentScenery]->wallname;
				}
				delete SceneryItems[CurrentScenery];
				SceneryItems.erase(SceneryItems.begin()+CurrentScenery);
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (SceneryItemsIterator = SceneryItems.begin();SceneryItemsIterator != SceneryItems.end();SceneryItemsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*SceneryItemsIterator)->name);
				}
			}
			break;
		case IDC_CLOSE:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK TextureManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_INITDIALOG:
		SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
		for (IconTexturesIterator = IconTextures.begin();IconTexturesIterator != IconTextures.end();IconTexturesIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconTexturesIterator)->name);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_ADD:
			CurrentIconTexture = -1;
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTUREEDIT), hwnd, TextureEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (IconTexturesIterator = IconTextures.begin();IconTexturesIterator != IconTextures.end();IconTexturesIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconTexturesIterator)->name);
				}
			}
			break;
		case IDC_EDIT:
			CurrentIconTexture = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTUREEDIT), hwnd, TextureEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (IconTexturesIterator = IconTextures.begin();IconTexturesIterator != IconTextures.end();IconTexturesIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconTexturesIterator)->name);
				}
			}
			break;
		case IDC_DEL:
			if (MessageBox(hwnd,"Are you sure you want to delete?","Are You Sure",MB_YESNO|MB_ICONQUESTION) == IDYES)
			{
				save = true;
				CurrentIconTexture = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
				delete IconTextures[CurrentIconTexture]->filename;
				delete IconTextures[CurrentIconTexture]->name;
				delete IconTextures[CurrentIconTexture];
				IconTextures.erase(IconTextures.begin()+CurrentIconTexture);
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (IconTexturesIterator = IconTextures.begin();IconTexturesIterator != IconTextures.end();IconTexturesIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconTexturesIterator)->name);
				}
			}
			break;
		case IDC_CLOSE:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK GSIManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_INITDIALOG:
		SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
		for (IconsIterator = Icons.begin();IconsIterator != Icons.end();IconsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconsIterator)->name);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_ADD:
			CurrentIcon = -1;
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIEDIT), hwnd, GSIEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (IconsIterator = Icons.begin();IconsIterator != Icons.end();IconsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconsIterator)->name);
				}
			}
			break;
		case IDC_EDIT:
			CurrentIcon = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIEDIT), hwnd, GSIEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (IconsIterator = Icons.begin();IconsIterator != Icons.end();IconsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconsIterator)->name);
				}
			}
			break;
		case IDC_DEL:
			if (MessageBox(hwnd,"Are you sure you want to delete?","Are You Sure",MB_YESNO|MB_ICONQUESTION) == IDYES)
			{
				save = true;
				CurrentIcon = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
				delete Icons[CurrentIcon]->name;
				delete Icons[CurrentIcon]->texture;
				delete Icons[CurrentIcon];
				Icons.erase(Icons.begin()+CurrentIcon);
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (IconsIterator = Icons.begin();IconsIterator != Icons.end();IconsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*IconsIterator)->name);
				}
			}
			break;
		case IDC_CLOSE:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
int compare(const void *s1,const void *s2)
{
	return stricmp(((SymbolStruct *)s1)->Symbol,((SymbolStruct *)s2)->Symbol);
}
BOOL CALLBACK TextManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_INITDIALOG:
		SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
		for (TextStringsIterator = TextStrings.begin();TextStringsIterator != TextStrings.end();TextStringsIterator++)
		{
			SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*TextStringsIterator)->name);
		}
		break;
	case WM_CLOSE:
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_ADD:
			CurrentTextString = -1;
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTEDIT), hwnd, TextEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (TextStringsIterator = TextStrings.begin();TextStringsIterator != TextStrings.end();TextStringsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*TextStringsIterator)->name);
				}
			}
			break;
		case IDC_EDIT:
			CurrentTextString = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
			if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTEDIT), hwnd, TextEditDlgProc) == 1)
			{
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (TextStringsIterator = TextStrings.begin();TextStringsIterator != TextStrings.end();TextStringsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*TextStringsIterator)->name);
				}
			}
			break;
		case IDC_DEL:
			if (MessageBox(hwnd,"Are you sure you want to delete?","Are You Sure",MB_YESNO|MB_ICONQUESTION) == IDYES)
			{
				save = true;
				CurrentTextString = SendDlgItemMessage(hwnd,IDC_LIST,LB_GETCURSEL,0,0);
				delete TextStrings[CurrentTextString]->name;
				delete TextStrings[CurrentTextString]->text;
				delete TextStrings[CurrentTextString];
				TextStrings.erase(TextStrings.begin()+CurrentTextString);
				SendDlgItemMessage(hwnd,IDC_LIST,LB_RESETCONTENT,0,0);
				for (TextStringsIterator = TextStrings.begin();TextStringsIterator != TextStrings.end();TextStringsIterator++)
				{
					SendDlgItemMessage(hwnd,IDC_LIST,LB_ADDSTRING,0,(LPARAM)(*TextStringsIterator)->name);
				}
			}
			break;
		case IDC_CLOSE:
			EndDialog(hwnd, 0);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
BOOL CALLBACK MainDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	OPENFILENAME ofn;
	char sfile[MAX_PATH] = "";
	switch(Message)
	{
	case WM_INITDIALOG:
		save = false;
		SendDlgItemMessage(hwnd,IDC_THEME,LB_ADDSTRING,0,(LPARAM)"Generic");
		SendDlgItemMessage(hwnd,IDC_THEME,LB_ADDSTRING,0,(LPARAM)"Western");
		SendDlgItemMessage(hwnd,IDC_THEME,LB_ADDSTRING,0,(LPARAM)"Spooky");
		SendDlgItemMessage(hwnd,IDC_THEME,LB_ADDSTRING,0,(LPARAM)"Sci-fi");
		SendDlgItemMessage(hwnd,IDC_THEME,LB_ADDSTRING,0,(LPARAM)"Adventure");
		break;
	case WM_CLOSE:
		if (save == true)
		{
			if (MessageBox(hwnd,"Unsaved changes, continue?","Unsaved changes",MB_YESNO) == IDNO)
			{
				break;
			}
		}
		EndDialog(hwnd, 0);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case ID_FILE_EXIT:
			if (save == true)
			{
				if (MessageBox(hwnd,"Unsaved changes, continue?","Unsaved changes",MB_YESNO) == IDNO)
				{
					break;
				}
			}
			EndDialog(hwnd, 0);
			break;
		case ID_FILE_NEWTHEME:
			if (save == true)
			{
				if (MessageBox(hwnd,"Unsaved changes, continue?","Unsaved changes",MB_YESNO) == IDNO)
				{
					break;
				}
			}
			save = false;
			for (unsigned long i = 0;i < SceneryItems.size();i++)
			{
				delete SceneryItems[i]->name;
				delete SceneryItems[i]->ovl;
				delete SceneryItems[i]->location;
				delete SceneryItems[i]->icon;
				if (SceneryItems[i]->wallicon != 0)
				{
					delete SceneryItems[i]->wallicon;
					delete SceneryItems[i]->wallname;
				}
				delete SceneryItems[i];
			}
			for (i = 0;i < IconTextures.size();i++)
			{
				delete IconTextures[i]->filename;
				delete IconTextures[i]->name;
				delete IconTextures[i];
			}
			for (i = 0;i < Icons.size();i++)
			{
				delete Icons[i]->name;
				delete Icons[i]->texture;
				delete Icons[i];
			}
			for (i = 0;i < TextStrings.size();i++)
			{
				delete TextStrings[i]->name;
				delete TextStrings[i]->text;
				delete TextStrings[i];
			}
			SetDlgItemText(hwnd,IDC_THEMENAME,"");
			SendDlgItemMessage(hwnd,IDC_THEME,LB_SETCURSEL,-1,0);
			TextStrings.clear();
			Icons.clear();
			IconTextures.clear();
			SceneryItems.clear();
			break;
		case ID_FILE_SAVETHEME:
			ZeroMemory(&ofn, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = hwnd;
			ofn.lpstrFilter = "Theme Files (*.thm)\0*.thm\0";
			ofn.lpstrFile = sfile;
			ofn.lpstrTitle = "Save Theme file";
			ofn.lpstrDefExt = ".thm";
			ofn.nMaxFile = MAX_PATH;
			ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT;
			if (GetSaveFileName(&ofn))
			{
				save = false;
				unsigned long style = SendDlgItemMessage(hwnd,IDC_THEME,LB_GETCURSEL,0,0);
				unsigned long len = GetWindowTextLength(GetDlgItem(hwnd,IDC_THEMENAME));
				if ((SceneryItems.size() == 0) || (style == -1) || (len == 0))
				{
					MessageBox(hwnd,"Unable to save because of missing data","Error",MB_OK);
				}
				else
				{
					FILE *f = fopen(sfile,"wb");
					fwrite(&style,4,1,f);
					len++;
					char *temp = new char[len];
					GetDlgItemText(hwnd,IDC_THEMENAME,temp,len);
					fwrite(&len,4,1,f);
					fwrite(temp,len,1,f);
					delete temp;
					len = TextStrings.size();
					fwrite(&len,4,1,f);
					for (i = 0;i < TextStrings.size();i++)
					{
						len = strlen(TextStrings[i]->name);
						len++;
						fwrite(&len,4,1,f);
						fwrite(TextStrings[i]->name,len,1,f);
						len = strlen(TextStrings[i]->text);
						len++;
						fwrite(&len,4,1,f);
						fwrite(TextStrings[i]->text,len,1,f);
					}
					len = IconTextures.size();
					fwrite(&len,4,1,f);
					for (i = 0;i < IconTextures.size();i++)
					{
						len = strlen(IconTextures[i]->name);
						len++;
						fwrite(&len,4,1,f);
						fwrite(IconTextures[i]->name,len,1,f);
						len = strlen(IconTextures[i]->filename);
						len++;
						fwrite(&len,4,1,f);
						fwrite(IconTextures[i]->filename,len,1,f);
					}
					len = Icons.size();
					fwrite(&len,4,1,f);
					for (i = 0;i < Icons.size();i++)
					{
						len = strlen(Icons[i]->name);
						len++;
						fwrite(&len,4,1,f);
						fwrite(Icons[i]->name,len,1,f);
						len = strlen(Icons[i]->texture);
						len++;
						fwrite(&len,4,1,f);
						fwrite(Icons[i]->texture,len,1,f);
						fwrite(&Icons[i]->top,4,1,f);
						fwrite(&Icons[i]->left,4,1,f);
						fwrite(&Icons[i]->bottom,4,1,f);
						fwrite(&Icons[i]->right,4,1,f);
					}
					len = SceneryItems.size();
					fwrite(&len,4,1,f);
					for (unsigned long i = 0;i < SceneryItems.size();i++)
					{
						Scenery *s = SceneryItems[i];
						len = strlen(s->ovl);
						len++;
						fwrite(&len,4,1,f);
						fwrite(s->ovl,len,1,f);
						if (s->wallicon != 0)
						{
							len = strlen(s->wallicon);
							len++;
							fwrite(&len,4,1,f);
							fwrite(s->wallicon,len,1,f);
						}
						else
						{
							len = 0;							
							fwrite(&len,4,1,f);
						}
						len = strlen(s->icon);
						len++;
						fwrite(&len,4,1,f);
						fwrite(s->icon,len,1,f);
						len = strlen(s->name);
						len++;
						fwrite(&len,4,1,f);
						fwrite(s->name,len,1,f);
						if (s->wallname != 0)
						{
							len = strlen(s->wallname);
							len++;
							fwrite(&len,4,1,f);
							fwrite(s->wallname,len,1,f);
						}
						else
						{
							len = 0;							
							fwrite(&len,4,1,f);
						}
						len = strlen(s->location);
						len++;
						fwrite(&len,4,1,f);
						fwrite(s->location,len,1,f);
						fwrite(&s->cost,4,1,f);
						fwrite(&s->refund,4,1,f);
						fwrite(&s->type,4,1,f);
						fwrite(&s->unk4,4,1,f);
						fwrite(&s->supportstype,4,1,f);
						fwrite(&s->unk17,4,1,f);
						fwrite(&s->sizex,4,1,f);
						fwrite(&s->sizey,4,1,f);
						fwrite(&s->sizeflag,4,1,f);
						fwrite(&s->extra,4,1,f);
						fwrite(&s->size.x,4,1,f);
						fwrite(&s->size.y,4,1,f);
						fwrite(&s->size.z,4,1,f);
						fwrite(&s->position.x,4,1,f);
						fwrite(&s->position.y,4,1,f);
						fwrite(&s->position.z,4,1,f);
						fwrite(&s->flag1,4,1,f);
						fwrite(&s->flag2,4,1,f);
						fwrite(&s->flag3,4,1,f);
						fwrite(&s->flag4,4,1,f);
						fwrite(&s->flag5,4,1,f);
						fwrite(&s->flag6,4,1,f);
						fwrite(&s->flag7,4,1,f);
						fwrite(&s->flag8,4,1,f);
						fwrite(&s->flag9,4,1,f);
						fwrite(&s->flag10,4,1,f);
						fwrite(&s->flag11,4,1,f);
						fwrite(&s->flag12,4,1,f);
						fwrite(&s->flag13,4,1,f);
						fwrite(&s->flag14,4,1,f);
						fwrite(&s->flag15,4,1,f);
						fwrite(&s->flag16,4,1,f);
						fwrite(&s->flag17,4,1,f);
						fwrite(&s->flag18,4,1,f);
						fwrite(&s->flag19,4,1,f);
						fwrite(&s->flag20,4,1,f);
						fwrite(&s->flag21,4,1,f);
						fwrite(&s->flag22,4,1,f);
						fwrite(&s->flag23,4,1,f);
						fwrite(&s->flag24,4,1,f);
						fwrite(&s->flag25,4,1,f);
						fwrite(&s->flag26,4,1,f);
						fwrite(&s->flag27,4,1,f);
						fwrite(&s->flag28,4,1,f);
						fwrite(&s->flag29,4,1,f);
						fwrite(&s->flag30,4,1,f);
						fwrite(&s->flag31,4,1,f);
						fwrite(&s->flag32,4,1,f);
						fwrite(&s->flag33,4,1,f);
						fwrite(&s->flag34,4,1,f);
						fwrite(&s->flag35,4,1,f);
						fwrite(&s->flag36,4,1,f);
						fwrite(&s->flag37,4,1,f);
						fwrite(&s->flag38,4,1,f);
						fwrite(&s->flag39,4,1,f);
						fwrite(&s->flag40,4,1,f);
						fwrite(&s->flag41,4,1,f);
						fwrite(&s->flag42,4,1,f);
						fwrite(&s->flag43,4,1,f);
						fwrite(&s->flag44,4,1,f);
						fwrite(&s->flag45,4,1,f);
						fwrite(&s->flag46,4,1,f);
						fwrite(&s->flag47,4,1,f);
						fwrite(&s->flag48,4,1,f);
						fwrite(&s->flag49,4,1,f);
						fwrite(&s->flag50,4,1,f);
						fwrite(&s->flag51,4,1,f);
						fwrite(&s->flag52,4,1,f);
						fwrite(&s->flag53,4,1,f);
						fwrite(&s->flag54,4,1,f);
						fwrite(&s->flag55,4,1,f);
						fwrite(&s->flag56,4,1,f);
						fwrite(&s->flag57,4,1,f);
						fwrite(&s->flag58,4,1,f);
						fwrite(&s->flag59,4,1,f);
						fwrite(&s->flag60,4,1,f);
						fwrite(&s->flag61,4,1,f);
						fwrite(&s->flag62,4,1,f);
						fwrite(&s->flag63,4,1,f);
						fwrite(&s->flag64,4,1,f);
						fwrite(&s->unk31,4,1,f);
						fwrite(&s->unk32,4,1,f);
						fwrite(&s->unk33,4,1,f);
						fwrite(&s->dunk2,4,1,f);
						fwrite(&s->dunk3,4,1,f);
						fwrite(&s->dunk4,4,1,f);
						fwrite(&s->dunk5,4,1,f);
					}
					fclose(f);
				}
			}
			break;
		case ID_FILE_OPENTHEME:
			if (save == true)
			{
				if (MessageBox(hwnd,"Unsaved changes, continue?","Unsaved changes",MB_YESNO) == IDNO)
				{
					break;
				}
			}
			ZeroMemory(&ofn, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = hwnd;
			ofn.lpstrFilter = "Theme Files (*.thm)\0*.thm\0All Files (*.*)\0*.*\0";
			ofn.lpstrFile = sfile;
			ofn.lpstrTitle = "Open Theme File";
			ofn.nMaxFile = MAX_PATH;
			ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT | OFN_NOCHANGEDIR;
			if (GetOpenFileName(&ofn))
			{
				save = false;
				for (unsigned long i = 0;i < SceneryItems.size();i++)
				{
					delete SceneryItems[i]->name;
					delete SceneryItems[i]->ovl;
					delete SceneryItems[i]->location;
					delete SceneryItems[i]->icon;
					if (SceneryItems[i]->wallicon != 0)
					{
						delete SceneryItems[i]->wallicon;
						delete SceneryItems[i]->wallname;
					}
					delete SceneryItems[i];
				}
				for (i = 0;i < IconTextures.size();i++)
				{
					delete IconTextures[i]->filename;
					delete IconTextures[i]->name;
					delete IconTextures[i];
				}
				for	(i = 0;i < Icons.size();i++)
				{
					delete Icons[i]->name;
					delete Icons[i]->texture;
					delete Icons[i];
				}
				for (i = 0;i < TextStrings.size();i++)
				{
					delete TextStrings[i]->name;
					delete TextStrings[i]->text;
					delete TextStrings[i];
				}
				TextStrings.clear();
				Icons.clear();
				IconTextures.clear();
				SceneryItems.clear();
				FILE *f = fopen(sfile,"rb");
				unsigned long style;
				unsigned long len;
				char *temp;
				fread(&style,4,1,f);
				fread(&len,4,1,f);
				temp = new char[len];
				fread(temp,len,1,f);
				SendDlgItemMessage(hwnd,IDC_THEME,LB_SETCURSEL,style,0);
				SetDlgItemText(hwnd,IDC_THEMENAME,temp);
				delete temp;
				unsigned long len2;
				fread(&len,4,1,f);
				for (unsigned long i = 0;i < len;i++)
				{
					Text *t = new Text;
					fread(&len2,4,1,f);
					t->name = new char[len2];
					fread(t->name,len2,1,f);
					fread(&len2,4,1,f);
					t->text = new char[len2];
					fread(t->text,len2,1,f);
					TextStrings.push_back(t);					
				}
				fread(&len,4,1,f);
				for (i = 0;i < len;i++)
				{
					IconTexture *t = new IconTexture;
					fread(&len2,4,1,f);
					t->name = new char[len2];
					fread(t->name,len2,1,f);
					fread(&len2,4,1,f);
					t->filename = new char[len2];
					fread(t->filename,len2,1,f);
					IconTextures.push_back(t);					
				}
				fread(&len,4,1,f);
				for (i = 0;i < len;i++)
				{
					Icon *t = new Icon;
					fread(&len2,4,1,f);
					t->name = new char[len2];
					fread(t->name,len2,1,f);
					fread(&len2,4,1,f);
					t->texture = new char[len2];
					fread(t->texture,len2,1,f);
					Icons.push_back(t);
					fread(&t->top,4,1,f);
					fread(&t->left,4,1,f);
					fread(&t->bottom,4,1,f);
					fread(&t->right,4,1,f);
				}
				fread(&len,4,1,f);
				for (i = 0;i < len;i++)
				{
					Scenery *t = new Scenery;
					fread(&len2,4,1,f);
					t->ovl = new char[len2];
					fread(t->ovl,len2,1,f);
					fread(&len2,4,1,f);
					if (len2 != 0)
					{
						t->wallicon = new char[len2];
						fread(t->wallicon,len2,1,f);
					}
					else
					{
						t->wallicon = 0;
					}
					fread(&len2,4,1,f);
					t->icon = new char[len2];
					fread(t->icon,len2,1,f);
					fread(&len2,4,1,f);
					t->name = new char[len2];
					fread(t->name,len2,1,f);
					fread(&len2,4,1,f);
					if (len2 != 0)
					{
						t->wallname = new char[len2];
						fread(t->wallname,len2,1,f);
					}
					else
					{
						t->wallname = 0;
					}
					fread(&len2,4,1,f);
					t->location = new char[len2];
					fread(t->location,len2,1,f);
					fread(&t->cost,4,1,f);
					fread(&t->refund,4,1,f);
					fread(&t->type,4,1,f);
					fread(&t->unk4,4,1,f);
					fread(&t->supportstype,4,1,f);
					fread(&t->unk17,4,1,f);
					fread(&t->sizex,4,1,f);
					fread(&t->sizey,4,1,f);
					fread(&t->sizeflag,4,1,f);
					fread(&t->extra,4,1,f);
					fread(&t->size.x,4,1,f);
					fread(&t->size.y,4,1,f);
					fread(&t->size.z,4,1,f);
					fread(&t->position.x,4,1,f);
					fread(&t->position.y,4,1,f);
					fread(&t->position.z,4,1,f);
					fread(&t->flag1,4,1,f);
					fread(&t->flag2,4,1,f);
					fread(&t->flag3,4,1,f);
					fread(&t->flag4,4,1,f);
					fread(&t->flag5,4,1,f);
					fread(&t->flag6,4,1,f);
					fread(&t->flag7,4,1,f);
					fread(&t->flag8,4,1,f);
					fread(&t->flag9,4,1,f);
					fread(&t->flag10,4,1,f);
					fread(&t->flag11,4,1,f);
					fread(&t->flag12,4,1,f);
					fread(&t->flag13,4,1,f);
					fread(&t->flag14,4,1,f);
					fread(&t->flag15,4,1,f);
					fread(&t->flag16,4,1,f);
					fread(&t->flag17,4,1,f);
					fread(&t->flag18,4,1,f);
					fread(&t->flag19,4,1,f);
					fread(&t->flag20,4,1,f);
					fread(&t->flag21,4,1,f);
					fread(&t->flag22,4,1,f);
					fread(&t->flag23,4,1,f);
					fread(&t->flag24,4,1,f);
					fread(&t->flag25,4,1,f);
					fread(&t->flag26,4,1,f);
					fread(&t->flag27,4,1,f);
					fread(&t->flag28,4,1,f);
					fread(&t->flag29,4,1,f);
					fread(&t->flag30,4,1,f);
					fread(&t->flag31,4,1,f);
					fread(&t->flag32,4,1,f);
					fread(&t->flag33,4,1,f);
					fread(&t->flag34,4,1,f);
					fread(&t->flag35,4,1,f);
					fread(&t->flag36,4,1,f);
					fread(&t->flag37,4,1,f);
					fread(&t->flag38,4,1,f);
					fread(&t->flag39,4,1,f);
					fread(&t->flag40,4,1,f);
					fread(&t->flag41,4,1,f);
					fread(&t->flag42,4,1,f);
					fread(&t->flag43,4,1,f);
					fread(&t->flag44,4,1,f);
					fread(&t->flag45,4,1,f);
					fread(&t->flag46,4,1,f);
					fread(&t->flag47,4,1,f);
					fread(&t->flag48,4,1,f);
					fread(&t->flag49,4,1,f);
					fread(&t->flag50,4,1,f);
					fread(&t->flag51,4,1,f);
					fread(&t->flag52,4,1,f);
					fread(&t->flag53,4,1,f);
					fread(&t->flag54,4,1,f);
					fread(&t->flag55,4,1,f);
					fread(&t->flag56,4,1,f);
					fread(&t->flag57,4,1,f);
					fread(&t->flag58,4,1,f);
					fread(&t->flag59,4,1,f);
					fread(&t->flag60,4,1,f);
					fread(&t->flag61,4,1,f);
					fread(&t->flag62,4,1,f);
					fread(&t->flag63,4,1,f);
					fread(&t->flag64,4,1,f);
					fread(&t->unk31,4,1,f);
					fread(&t->unk32,4,1,f);
					fread(&t->unk33,4,1,f);
					fread(&t->dunk2,4,1,f);
					fread(&t->dunk3,4,1,f);
					fread(&t->dunk4,4,1,f);
					fread(&t->dunk5,4,1,f);
					SceneryItems.push_back(t);
				}
			}
			break;
		case ID_FILE_INSTALL:
			unsigned long styleval,len;
			styleval = SendDlgItemMessage(hwnd,IDC_THEME,LB_GETCURSEL,0,0);
			len = GetWindowTextLength(GetDlgItem(hwnd,IDC_THEMENAME));
			if ((SceneryItems.size() == 0) || (styleval == -1) || (len == 0))
			{
				MessageBox(hwnd,"Unable to install because of missing data","Error",MB_OK);
			}
			else
			{
				HKEY key;
				LONG res = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\{907B4640-266B-4A21-92FB-CD1A86CD0F63}",0,KEY_QUERY_VALUE,&key);
				char InstallLocation[MAX_PATH] = "";
				char Location[MAX_PATH] = "";
				unsigned long length;
				res = RegQueryValueEx(key,"InstallLocation",0,NULL,(LPBYTE)InstallLocation,&length);
				res = RegCloseKey(key);
				len++;
				char *theme = new char[len];
				GetDlgItemText(hwnd,IDC_THEMENAME,theme,len);
				unsigned long stringcount = 0;
				unsigned long stringtablesize = 0;
				unsigned long textstringsize = 0;
				unsigned long loadersize = 0;
				unsigned long symbolsize = 0;
				unsigned long symbolrefsize = 0;
				unsigned long flicsize = 0;
				unsigned long texturesize = 0;
				unsigned long gsisize = 0;
				unsigned long sidsize = 0;
				unsigned char *stringtable;
				unsigned char *textstrings;
				unsigned char *loaders;
				unsigned char *symbols;
				unsigned char *symrefs;
				unsigned char *flics;
				unsigned char *textures;
				unsigned char *gsis;
				unsigned char *sids;
				symbolsize += sizeof(SymbolStruct);
				symbolsize += sizeof(SymbolStruct);
				symbolsize += sizeof(SymbolStruct);
				symbolsize += sizeof(SymbolStruct);
				for (i = 0;i < TextStrings.size();i++)
				{
					stringtablesize += strlen(TextStrings[i]->name)+5;
					textstringsize += (strlen(TextStrings[i]->text)+1)*2;
					loadersize += sizeof(LoaderStruct);
					symbolsize += sizeof(SymbolStruct);
					stringcount++;
				}
				for (i = 0;i < IconTextures.size();i++)
				{
					stringtablesize += strlen(IconTextures[i]->name)+5;
					flicsize += sizeof(FlicStruct);
					flicsize += 4;
					texturesize += sizeof(TextureStruct2);
					texturesize += sizeof(TextureStruct);
					loadersize += sizeof(LoaderStruct);
					loadersize += sizeof(LoaderStruct);
					symbolsize += sizeof(SymbolStruct);
					symbolrefsize += sizeof(SymbolRefStruct);
				}
				for (i = 0;i < Icons.size();i++)
				{
					stringtablesize += strlen(Icons[i]->name)+5;
					gsisize += sizeof(GUISkinItemPos);
					gsisize += sizeof(GUISkinItem);
					loadersize += sizeof(LoaderStruct);
					symbolsize += sizeof(SymbolStruct);
					symbolrefsize += sizeof(SymbolRefStruct);
				}
				for (unsigned long i = 0;i < SceneryItems.size();i++)
				{
					char fname[_MAX_FNAME];
					_splitpath(SceneryItems[i]->ovl,NULL,NULL,fname,NULL);
					strchr(fname,'.')[0] = 0;
					stringtablesize += strlen(fname)+5;
					stringtablesize += strlen(fname)+5;
					stringtablesize += strlen(stdstyle) + strlen(theme) + 1 + strlen(SceneryItems[i]->location) + 1 + strlen(fname) + 1;
					loadersize += sizeof(LoaderStruct);
					symbolsize += sizeof(SymbolStruct);
					symbolrefsize += sizeof(SymbolRefStruct);
					symbolrefsize += sizeof(SymbolRefStruct);
					symbolrefsize += sizeof(SymbolRefStruct);
					if (SceneryItems[i]->wallicon != 0)
					{
						symbolrefsize += sizeof(SymbolRefStruct);
						symbolrefsize += sizeof(SymbolRefStruct);
					}
					sidsize += sizeof(SceneryItem);
					sidsize += sizeof(SceneryItemData);
					sidsize += 8;
					if (SceneryItems[i]->extra != 0)
					{
						sidsize += sizeof(SceneryParams);
					}
				}
				stringtablesize += 434;
				stringtable = new unsigned char[stringtablesize];
				symbols = new unsigned char[symbolsize];
				loaders = new unsigned char[loadersize];
				symrefs = new unsigned char[symbolrefsize];
				textstrings = new unsigned char[textstringsize];
				flics = new unsigned char[flicsize];
				textures = new unsigned char[texturesize];
				gsis = new unsigned char[gsisize];
				sids = new unsigned char[sidsize];
				unsigned char *sidsx = sids;
				OutOvlCount = 1;
				sprintf(OutOvlFiles[1].FileName,"%s\\%s%s\\",InstallLocation,stdstyle,theme);
				SHCreateDirectoryEx(hwnd,OutOvlFiles[1].FileName,NULL);
				strcat(OutOvlFiles[1].FileName,"style.unique.ovl");
				File Type0Files[1];
				File Type2Files[8];
				Type0Files[0].data = (unsigned long *)stringtable;
				Type0Files[0].reloffset = 0;
				Type0Files[0].size = stringtablesize;
				Type2Files[0].data = (unsigned long *)symbols;
				Type2Files[0].reloffset = Type0Files[0].size;
				Type2Files[0].size = symbolsize;
				Type2Files[1].data = (unsigned long *)loaders;
				Type2Files[1].reloffset = Type2Files[0].reloffset + Type2Files[0].size;
				Type2Files[1].size = loadersize;
				Type2Files[2].data = (unsigned long *)symrefs;
				Type2Files[2].reloffset = Type2Files[1].reloffset + Type2Files[1].size;
				Type2Files[2].size = symbolrefsize;
				Type2Files[3].data = (unsigned long *)textstrings;
				Type2Files[3].reloffset = Type2Files[2].reloffset + Type2Files[2].size;
				Type2Files[3].size = textstringsize;
				Type2Files[4].data = (unsigned long *)flics;
				Type2Files[4].reloffset = Type2Files[3].reloffset + Type2Files[3].size;
				Type2Files[4].size = flicsize;
				Type2Files[5].data = (unsigned long *)textures;
				Type2Files[5].reloffset = Type2Files[4].reloffset + Type2Files[4].size;
				Type2Files[5].size = texturesize;
				Type2Files[6].data = (unsigned long *)gsis;
				Type2Files[6].reloffset = Type2Files[5].reloffset + Type2Files[5].size;
				Type2Files[6].size = gsisize;
				Type2Files[7].data = (unsigned long *)sids;
				Type2Files[7].reloffset = Type2Files[6].reloffset + Type2Files[6].size;
				Type2Files[7].size = sidsize;
				OutOvlFiles[0].Types[0].count = 1;
				OutOvlFiles[0].Types[0].dataptr = Type0Files;
				OutOvlFiles[0].Types[0].reloffset = 0;
				OutOvlFiles[0].Types[0].size = stringtablesize;
				OutOvlFiles[1].Types[0].count = 0;
				OutOvlFiles[1].Types[0].dataptr = 0;
				OutOvlFiles[1].Types[0].reloffset = stringtablesize;
				OutOvlFiles[1].Types[0].size = 0;
				OutOvlFiles[1].Types[1].count = 0;
				OutOvlFiles[1].Types[1].dataptr = 0;
				OutOvlFiles[1].Types[1].size = 0;
				OutOvlFiles[1].Types[1].reloffset = stringtablesize;
				OutOvlFiles[1].Types[2].reloffset = OutOvlFiles[0].Types[1].reloffset;
				OutOvlFiles[1].Types[2].dataptr = Type2Files;
				OutOvlFiles[1].Types[2].count = 8;
				OutOvlFiles[1].Types[2].size = symbolsize+loadersize+symbolrefsize+textstringsize+flicsize+texturesize+gsisize+sidsize;
				OutOvlFiles[1].Types[3].count = 0;
				OutOvlFiles[1].Types[3].dataptr = 0;
				OutOvlFiles[1].Types[3].reloffset = OutOvlFiles[0].Types[2].reloffset+OutOvlFiles[0].Types[2].size;
				OutOvlFiles[1].Types[3].size = 0;
				OutOvlFiles[1].Types[4].count = 0;
				OutOvlFiles[1].Types[4].dataptr = 0;
				OutOvlFiles[1].Types[4].reloffset = OutOvlFiles[0].Types[2].reloffset+OutOvlFiles[0].Types[2].size;
				OutOvlFiles[1].Types[4].size = 0;
				OutOvlFiles[1].Types[5].count = 0;
				OutOvlFiles[1].Types[5].dataptr = 0;
				OutOvlFiles[1].Types[5].reloffset = OutOvlFiles[0].Types[2].reloffset+OutOvlFiles[0].Types[2].size;
				OutOvlFiles[1].Types[5].size = 0;
				OutOvlFiles[1].Types[6].count = 0;
				OutOvlFiles[1].Types[6].dataptr = 0;
				OutOvlFiles[1].Types[6].reloffset = OutOvlFiles[0].Types[2].reloffset+OutOvlFiles[0].Types[2].size;
				OutOvlFiles[1].Types[6].size = 0;
				OutOvlFiles[1].Types[7].count = 0;
				OutOvlFiles[1].Types[7].dataptr = 0;
				OutOvlFiles[1].Types[7].reloffset = OutOvlFiles[0].Types[2].reloffset+OutOvlFiles[0].Types[2].size;
				OutOvlFiles[1].Types[7].size = 0;
				OutOvlFiles[1].Types[8].count = 0;
				OutOvlFiles[1].Types[8].dataptr = 0;
				OutOvlFiles[1].Types[8].reloffset = OutOvlFiles[0].Types[2].reloffset+OutOvlFiles[0].Types[2].size;
				OutOvlFiles[1].Types[8].size = 0;
				char *strings = (char *)stringtable;
				wchar_t *text = (wchar_t *)textstrings;
				LoaderStruct *ldrs = (LoaderStruct *)loaders; //type 0 is text, 1 is texture, 2 is flic, 3 is GUISkinItem, 4 is SceneryItem
				SymbolStruct *syms = (SymbolStruct *)symbols;
				SymbolRefStruct *refs = (SymbolRefStruct *)symrefs;
				Tex *texstr = (Tex *)textures;
				Flic *flstr = (Flic *)flics;
				GSI *gsistr = (GSI *)gsis;
				unsigned char *sidstr = sids;
				memcpy(strings,stdstrings,434);
				char *StyleIndex = strings;
				char *GUIIcon = StyleIndex+(strlen(StyleIndex)+1);
				char *extra[10];
				char *extraparams[10];
				char *supports[24];
				char *mapblue;
				char *mapgreen;
				char *mapred;
				extra[1] = GUIIcon+(strlen(GUIIcon)+1);
				extra[2] = extra[1]+(strlen(extra[1])+1);
				extraparams[2] = extra[2]+(strlen(extra[2])+1);
				extra[3] = extraparams[2]+(strlen(extraparams[2])+1);
				extra[4] = extra[3]+(strlen(extra[3])+1);
				extra[5] = extra[4]+(strlen(extra[4])+1);
				extra[6] = extra[5]+(strlen(extra[5])+1);
				extra[7] = extra[6]+(strlen(extra[6])+1);
				extraparams[7] = extra[7]+(strlen(extra[7])+1);
				extra[8] = extraparams[7]+(strlen(extraparams[7])+1);
				extra[9] = extra[8]+(strlen(extra[8])+1);
				extraparams[9] = extra[9]+(strlen(extra[9])+1);
				supports[1] = extraparams[9]+(strlen(extraparams[9])+1);
				supports[2] = supports[1]+(strlen(supports[1])+1);
				supports[3] = supports[2]+(strlen(supports[2])+1);
				supports[4] = supports[3]+(strlen(supports[3])+1);
				supports[5] = supports[4]+(strlen(supports[4])+1);
				supports[6] = supports[5]+(strlen(supports[5])+1);
				supports[7] = supports[6]+(strlen(supports[6])+1);
				supports[8] = supports[7]+(strlen(supports[7])+1);
				supports[9] = supports[8]+(strlen(supports[8])+1);
				supports[10] = supports[9]+(strlen(supports[9])+1);
				supports[11] = supports[10]+(strlen(supports[10])+1);
				supports[12] = supports[11]+(strlen(supports[11])+1);
				supports[13] = supports[12]+(strlen(supports[12])+1);
				supports[14] = supports[13]+(strlen(supports[13])+1);
				supports[15] = supports[14]+(strlen(supports[14])+1);
				supports[16] = supports[15]+(strlen(supports[15])+1);
				supports[17] = supports[16]+(strlen(supports[16])+1);
				supports[18] = supports[17]+(strlen(supports[17])+1);
				supports[19] = supports[18]+(strlen(supports[18])+1);
				supports[20] = supports[19]+(strlen(supports[19])+1);
				supports[21] = supports[20]+(strlen(supports[20])+1);
				supports[22] = supports[21]+(strlen(supports[21])+1);
				supports[23] = supports[22]+(strlen(supports[22])+1);
				supports[0] = supports[23]+(strlen(supports[23])+1);
				extra[0] = supports[23]+(strlen(supports[23])+1);
				extraparams[0] = supports[23]+(strlen(supports[23])+1);
				extraparams[1] = supports[23]+(strlen(supports[23])+1);
				extraparams[3] = supports[23]+(strlen(supports[23])+1);
				extraparams[4] = supports[23]+(strlen(supports[23])+1);
				extraparams[5] = supports[23]+(strlen(supports[23])+1);
				extraparams[6] = supports[23]+(strlen(supports[23])+1);
				extraparams[8] = supports[23]+(strlen(supports[23])+1);
				mapblue = extraparams[0]+1;
				mapred = mapblue+strlen(mapblue)+1;
				mapgreen = mapred+strlen(mapred)+1;
				strings = mapgreen+strlen(mapgreen)+1;
				syms->Symbol = StyleIndex;
				syms->data = (unsigned long *)styleval;
				syms->unk = 0;
				syms++;
				syms->Symbol = mapred;
				syms->data = 0;
				syms->unk = 0;
				syms++;
				syms->Symbol = mapgreen;
				syms->data = 0;
				syms->unk = 0;
				syms++;
				syms->Symbol = mapblue;
				syms->data = 0;
				syms->unk = 0;
				syms++;
				for (i = 0;i < TextStrings.size();i++)
				{
					char *str = strings;
					strcpy(strings,TextStrings[i]->name);
					strings += strlen(TextStrings[i]->name);
					strcpy(strings,":txt");
					strings += 5;
					mbstowcs(text,TextStrings[i]->text,strlen(TextStrings[i]->text)+1);
					ldrs->data = (unsigned long *)text;
					ldrs->LoaderType = 0;
					ldrs->Sym = syms;
					ldrs->SymbolsToResolve = 0;
					ldrs->unk1 = 1;
					syms->data = (unsigned long *)text;
					syms->Symbol = str;
					syms->unk = 1;
					relocations.push((unsigned long *)&ldrs->data);
					relocations.push((unsigned long *)&ldrs->Sym);
					text += strlen(TextStrings[i]->text)+1;
					ldrs++;
					syms++;
				}
				for (i = 0;i < IconTextures.size();i++)
				{
					char *str = strings;
					strcpy(strings,IconTextures[i]->name);
					strings += strlen(IconTextures[i]->name);
					strcpy(strings,":tex");
					strings += 5;
					flstr->fl = &flstr->fl2;
					relocations.push((unsigned long *)&flstr->fl);
					flstr->fl2.FlicDataPtr = 0;
					flstr->fl2.unk1 = 1;
					flstr->fl2.unk2 = 1.0;
					texstr->t1.unk1 = 0x70007;
					texstr->t1.unk2 = 0x70007;
					texstr->t1.unk3 = 0x70007;
					texstr->t1.unk4 = 0x70007;
					texstr->t1.unk5 = 0x70007;
					texstr->t1.unk6 = 0x70007;
					texstr->t1.unk7 = 0x70007;
					texstr->t1.unk8 = 0x70007;
					texstr->t1.unk9 = 1;
					texstr->t1.unk10 = 8;
					texstr->t1.unk11 = 0x10;
					texstr->t1.TextureData = 0;
					texstr->t1.unk12 = 1;
					texstr->t1.ts2 = &texstr->t2;
					texstr->t1.Flic = &flstr->fl;
					texstr->t2.Texture = &texstr->t1;
					texstr->t2.Flic = &flstr->fl2;
					relocations.push((unsigned long *)&texstr->t1.ts2);
					relocations.push((unsigned long *)&texstr->t1.Flic);
					relocations.push((unsigned long *)&texstr->t2.Texture);
					relocations.push((unsigned long *)&texstr->t2.Flic);
					ldrs->data = (unsigned long *)&flstr->fl2;
					ldrs->LoaderType = 2;
					ldrs->Sym = 0;
					ldrs->SymbolsToResolve = 0;
					ldrs->unk1 = 1;
					relocations.push((unsigned long *)&ldrs->data);
					relocations.push((unsigned long *)&ldrs->Sym);
					ldrs++;
					ldrs->data = (unsigned long *)texstr;
					ldrs->LoaderType = 1;
					ldrs->Sym = syms;
					ldrs->SymbolsToResolve = 1;
					ldrs->unk1 = 0;
					syms->data = (unsigned long *)texstr;
					syms->Symbol = str;
					syms->unk = 1;
					refs->ldr = ldrs;
					refs->Symbol = GUIIcon;
					refs->reference = (unsigned long *)&texstr->t1.TextureData;
					relocations.push((unsigned long *)&ldrs->data);
					relocations.push((unsigned long *)&ldrs->Sym);
					relocations.push((unsigned long *)&refs->reference);
					relocations.push((unsigned long *)&refs->ldr);
					relocations.push((unsigned long *)&refs->Symbol);
					ldrs++;
					syms++;
					refs++;
				}
				for (i = 0;i < Icons.size();i++)
				{
					char *str = strings;
					strcpy(strings,Icons[i]->name);
					strings += strlen(Icons[i]->name);
					strcpy(strings,":gsi");
					strings += 5;
					gsistr->g1.pos = &gsistr->g2;
					gsistr->g1.tex = 0;
					gsistr->g1.unk1 = 0;
					gsistr->g1.unk2 = 0;
					gsistr->g2.top = Icons[i]->top;
					gsistr->g2.left = Icons[i]->left;
					gsistr->g2.bottom = Icons[i]->bottom;
					gsistr->g2.right = Icons[i]->right;
					ldrs->data = (unsigned long *)gsistr;
					ldrs->LoaderType = 3;
					ldrs->Sym = syms;
					ldrs->SymbolsToResolve = 1;
					ldrs->unk1 = 0;
					syms->data = (unsigned long *)gsistr;
					syms->Symbol = str;
					syms->unk = 1;
					refs->ldr = ldrs;
					unsigned long namel = strlen(Icons[i]->texture);
					for (char *s = (char *)stringtable;s < strings;s++)
					{
						if (memcmp(Icons[i]->texture,s,namel) == 0)
						{
							if (memcmp(s+namel,":tex",4) == 0)
							{
								refs->Symbol = s;
								break;
							}
						}
					}
					refs->reference = (unsigned long *)&gsistr->g1.tex;
					relocations.push((unsigned long *)&gsistr->g1.pos);
					relocations.push((unsigned long *)&ldrs->data);
					relocations.push((unsigned long *)&ldrs->Sym);
					relocations.push((unsigned long *)&refs->reference);
					relocations.push((unsigned long *)&refs->ldr);
					relocations.push((unsigned long *)&refs->Symbol);
					ldrs++;
					syms++;
					refs++;
					gsistr++;
				}
				for (unsigned long i = 0;i < SceneryItems.size();i++)
				{
					Scenery *sc = SceneryItems[i];
					char fname[_MAX_FNAME];
					_splitpath(sc->ovl,NULL,NULL,fname,NULL);
					strchr(fname,'.')[0] = 0;
					char *str1 = strings;
					strcpy(strings,fname);
					strings += strlen(fname);
					strcpy(strings,":sid");
					strings += 5;
					char *str2 = strings;
					strcpy(strings,fname);
					strings += strlen(fname);
					strcpy(strings,":svd");
					strings += 5;
					char *str3 = strings;
					strcpy(strings,stdstyle);
					strings += strlen(stdstyle);
					strcpy(strings,theme);
					strings += strlen(theme);
					strcpy(strings,"\\");
					strings++;
					strcpy(strings,sc->location);
					strings += strlen(sc->location);
					sprintf(Location,"%s\\%s",InstallLocation,str3);
					SHCreateDirectoryEx(hwnd,Location,NULL);
					strcpy(strings,"\\");
					strings++;
					strcpy(strings,fname);
					strings += strlen(fname) + 1;
					strcat(Location,"\\");
					strcat(Location,fname);
					strcat(Location,".common.ovl");
					CopyFile(sc->ovl,Location,FALSE);
					strcpy(strstr(Location,".common.ovl"),".unique.ovl");
					strcpy(strstr(sc->ovl,".common.ovl"),".unique.ovl");
					CopyFile(sc->ovl,Location,FALSE);
					strcpy(strstr(sc->ovl,".unique.ovl"),".common.ovl");
					SceneryItem *s = (SceneryItem *)sids;
                    sids += sizeof(SceneryItem);
                    SceneryItemData *s2 = (SceneryItemData *)sids;
                    sids += sizeof(SceneryItemData);
					long *s3 = (long *)sids;
					sids += 4;
					SceneryItemVisual **svd = (SceneryItemVisual **)sids;
					sids += 4;
					SceneryParams *s4 = 0;
					if (sc->extra != 0)
					{
						s4 = (SceneryParams *)sids;
						sids += sizeof(SceneryParams);
					}
					s->unk1 = 0;
					s->unkf1 = sc->flag1;
					s->unkf2 = sc->flag2;
					s->unkf3 = sc->flag3;
					s->unkf4 = sc->flag4;
					s->unkf5 = sc->flag5;
					s->unkf6 = sc->flag6;
					s->unkf7 = sc->flag7;
					s->unkf8 = sc->flag8;
					s->unkf9 = sc->flag9;
					s->unkf10 = sc->flag10;
					s->unkf11 = sc->flag11;
					s->unkf12 = sc->flag12;
					s->unkf13 = sc->flag13;
					s->unkf14 = sc->flag14;
					s->unkf15 = sc->flag15;
					s->unkf16 = sc->flag16;
					s->unkf17 = sc->flag17;
					s->unkf18 = sc->flag18;
					s->unkf19 = sc->flag19;
					s->unkf20 = sc->flag20;
					s->unkf21 = sc->flag21;
					s->unkf22 = sc->flag22;
					s->unkf23 = sc->flag23;
					s->unkf24 = sc->flag24;
					s->unkf25 = sc->flag25;
					s->unkf26 = sc->flag26;
					s->unkf27 = sc->flag27;
					s->unkf28 = sc->flag28;
					s->unkf29 = sc->flag29;
					s->unkf30 = sc->flag30;
					s->unkf31 = sc->flag31;
					s->unkf32 = sc->flag32;
					s->size = sc->sizeflag;
					s->unk4 = sc->unk4;
					s->xsquares = sc->sizex;
					s->ysquares = sc->sizey;
					s->data = s2;
					relocations.push((unsigned long *)&s->data);
					s->xpos = sc->position.x;
					s->ypos = sc->position.y;
					s->zpos = sc->position.z;
					s->xsize = sc->size.x;
					s->ysize = sc->size.y;
					s->zsize = sc->size.z;
					s->unk13 = 0;
					s->unk14 = 0;
					s->cost = sc->cost;
					s->refund = sc->refund;
					s->unk17 = 0xFFFFFFFF;
					if (sc->unk17 == 0)
					{
						s->unk17 = 0;
					}
					if (sc->unk17 == 1)
					{
						s->unk17 = 4;
					}
					if (sc->unk17 == 2)
					{
						s->unk17 = 5;
					}
					s->type = sc->type;
					s->supports = supports[sc->supportstype];
					relocations.push((unsigned long *)&s->supports);
					s->svdcount = 1;
					s->svd = svd;
					relocations.push((unsigned long *)&s->svd);
					s->gsi = 0;
					s->wallsetgsi = 0;
					s->wallsetname = 0;
					s->OvlName = str3;
					relocations.push((unsigned long *)&s->OvlName);
					if (s4 != 0)
					{
						s->params = s4;
						relocations.push((unsigned long *)&s->params);
						s->count = 1;
						s4->type = extra[sc->extra];
						s4->params = extraparams[sc->extra];
						relocations.push((unsigned long *)&s4->type);
						relocations.push((unsigned long *)&s4->params);
					}
					else
					{
						s->params = 0;
						s->count = 0;
					}
					s->unk25 = 0;
					s->unk26 = 0;
					s->Name = 0;
					s->unk27 = 0;
					s->unk28 = 0;
					s->unk29 = 0;
					s->unk30 = 0;
					s->unk31 = sc->unk31;
					s->unk32 = sc->unk32;
					s->unk33 = sc->unk33;
					s->unk34 = 0;
					s->unk35 = 0;
					s->unk36 = 0;
					s->unk37 = 0;
					s->unk38 = 0;
					s->unk39 = 0;
					s->unk40 = 0;
					s->unk41 = 0;
					s->unk42 = 0;
					s->unk43 = 0;
					s->unk44 = 0;
					s->unk45 = 0;
					s->unk46 = 0;
					s->unk47 = 0;
					s->unk48 = 0;
					s2->unkf33 = sc->flag33;
					s2->unkf34 = sc->flag34;
					s2->unkf35 = sc->flag35;
					s2->unkf36 = sc->flag36;
					s2->unkf37 = sc->flag37;
					s2->unkf38 = sc->flag38;
					s2->unkf39 = sc->flag39;
					s2->unkf40 = sc->flag40;
					s2->unkf41 = sc->flag41;
					s2->unkf42 = sc->flag42;
					s2->unkf43 = sc->flag43;
					s2->unkf44 = sc->flag44;
					s2->unkf45 = sc->flag45;
					s2->unkf46 = sc->flag46;
					s2->unkf47 = sc->flag47;
					s2->unkf48 = sc->flag48;
					s2->unkf49 = sc->flag49;
					s2->unkf50 = sc->flag50;
					s2->unkf51 = sc->flag51;
					s2->unkf52 = sc->flag52;
					s2->unkf53 = sc->flag53;
					s2->unkf54 = sc->flag54;
					s2->unkf55 = sc->flag55;
					s2->unkf56 = sc->flag56;
					s2->unkf57 = sc->flag57;
					s2->unkf58 = sc->flag58;
					s2->unkf59 = sc->flag59;
					s2->unkf60 = sc->flag60;
					s2->unkf61 = sc->flag61;
					s2->unkf62 = sc->flag62;
					s2->unkf63 = sc->flag63;
					s2->unkf64 = sc->flag64;
					s2->unk2 = sc->dunk2;
					s2->unk3 = sc->dunk3;
					s2->unk4 = s3;
					relocations.push((unsigned long *)&s2->unk4);
					s2->unk5 = sc->dunk5;
					*s3 = sc->dunk4;
					*svd = 0;
					ldrs->data = (unsigned long *)s;
					ldrs->LoaderType = 4;
					ldrs->Sym = syms;
					ldrs->unk1 = 0;
					syms->data = (unsigned long *)s;
					syms->Symbol = str1;
					syms->unk = 1;
					relocations.push((unsigned long *)&ldrs->data);
					relocations.push((unsigned long *)&ldrs->Sym);
					refs->ldr = ldrs;
					unsigned long namel = strlen(sc->icon);
					for (char *st = (char *)stringtable;st < strings;st++)
					{
						if (memcmp(sc->icon,st,namel) == 0)
						{
							if (memcmp(st+namel,":gsi",4) == 0)
							{
								refs->Symbol = st;
								break;
							}
						}
					}
					refs->reference = (unsigned long *)&s->gsi;
					relocations.push((unsigned long *)&refs->reference);
					relocations.push((unsigned long *)&refs->ldr);
					relocations.push((unsigned long *)&refs->Symbol);
					refs++;
					refs->ldr = ldrs;
					ldrs->SymbolsToResolve = 5;
					if (sc->wallicon != 0)
					{
						namel = strlen(sc->wallicon);
						for (char *st = (char *)stringtable;st < strings;st++)
						{
							if (memcmp(sc->wallicon,st,namel) == 0)
							{
								if (memcmp(st+namel,":gsi",4) == 0)
								{
									refs->Symbol = st;
									break;
								}
							}
						}
						refs->reference = (unsigned long *)&s->wallsetgsi;
						relocations.push((unsigned long *)&refs->reference);
						relocations.push((unsigned long *)&refs->ldr);
						relocations.push((unsigned long *)&refs->Symbol);
						refs++;
					}
					else
					{
						ldrs->SymbolsToResolve--;
						refs->Symbol = 0;
					}
					refs->ldr = ldrs;
					namel = strlen(sc->name);
					for (char *st = (char *)stringtable;st < strings;st++)
					{
						if (memcmp(sc->name,st,namel) == 0)
						{
							if (memcmp(st+namel,":txt",4) == 0)
							{
								refs->Symbol = st;
								break;
							}
						}
					}
					refs->reference = (unsigned long *)&s->Name;
					relocations.push((unsigned long *)&refs->reference);
					relocations.push((unsigned long *)&refs->ldr);
					relocations.push((unsigned long *)&refs->Symbol);
					refs++;
					if (sc->wallname != 0)
					{
						refs->ldr = ldrs;
						namel = strlen(sc->wallname);
						for (char *st = (char *)stringtable;st < strings;st++)
						{
							if (memcmp(sc->wallname,st,namel) == 0)
							{
								if (memcmp(st+namel,":txt",4) == 0)
								{
									refs->Symbol = st;
									break;
								}
							}
						}
						refs->reference = (unsigned long *)&s->wallsetname;
						relocations.push((unsigned long *)&refs->reference);
						relocations.push((unsigned long *)&refs->ldr);
						relocations.push((unsigned long *)&refs->Symbol);
						refs++;
					}
					else
					{
						ldrs->SymbolsToResolve--;
						refs->Symbol = 0;
					}
					refs->ldr = ldrs;
					refs->Symbol = str2;
					refs->reference = (unsigned long *)svd;
					relocations.push((unsigned long *)&refs->reference);
					relocations.push((unsigned long *)&refs->ldr);
					relocations.push((unsigned long *)&refs->Symbol);
					refs++;
					ldrs++;
					syms++;
				}
				SymbolStruct *syms2 = (SymbolStruct *)symbols;
				qsort(syms2,symbolsize/sizeof(SymbolStruct),sizeof(SymbolStruct),compare);
				for (i = 0;i < symbolsize/sizeof(SymbolStruct);i++)
				{
					relocations.push((unsigned long *)&(syms2[i].Symbol));
					if ((syms2[i].Symbol != mapblue) && (syms2[i].Symbol != mapgreen) && (syms2[i].Symbol != mapred) && (syms2[i].Symbol != StyleIndex))
					{
						relocations.push((unsigned long *)&(syms2[i].data));
					}
				}
				LoaderStruct *ldrs2 = (LoaderStruct *)loaders;
				for (i = 0;i < loadersize/sizeof(LoaderStruct);i++)
				{
					for (unsigned long j = 0;j < symbolsize/sizeof(SymbolStruct);j++)
					{
						if (ldrs2[i].data == syms2[j].data)
						{
							ldrs2[i].Sym = &syms2[j];
						}
					}
				}
				OvlHeader h;
				h.magic = 0x4B524746;
				h.References = 0;
				h.reserved = 0;
				h.version = 1;
				OvlHeader2 h2;
				h2.FileTypeCount = 5;
				h2.unk = 0;
				unsigned short len1 = 4;
				char *str1 = "FGDK";
				unsigned short len2 = 4;
				char *str2 = "Text";
				unsigned long type = 2;
				unsigned short len3 = 3;
				char *str3 = "txt";
				FILE *f = fopen(OutOvlFiles[1].FileName,"wb");
				fwrite(&h,sizeof(h),1,f);
				fwrite(&h2,sizeof(h2),1,f);
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "FGDK";
				len2 = 7;
				str2 = "Texture";
				type = 2;
				len3 = 3;
				str3 = "tex";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "FGDK";
				len2 = 4;
				str2 = "Flic";
				type = 2;
				len3 = 4;
				str3 = "flic";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "FGDK";
				len2 = 0xB;
				str2 = "GUISkinItem";
				type = 1;
				len3 = 3;
				str3 = "gsi";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "RCT3";
				len2 = 0xB;
				str2 = "SceneryItem";
				type = 1;
				len3 = 3;
				str3 = "sid";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				for (i = 0;i < 9;i++)
				{
					fwrite(&OutOvlFiles[1].Types[i].count,sizeof(OutOvlFiles[1].Types[i].count),1,f);
				}
				while (relocations.empty() == false)
				{
					unsigned long *relocation = relocations.front();
					relocations.pop();
					unsigned long *reloc = *(unsigned long **)relocation;
					if (reloc != 0)
					{
						unsigned long fixup = DoRelocation(reloc);
						if (fixup == 0xFFFFFFFF)
						{
							fixup = 0;
						}
						*relocation = fixup;
						unsigned long fixup2 = DoRelocation(relocation);
						fixups.push(fixup2);
					}
				}
				for (i = 0;i < 9;i++)
				{
					for (unsigned long j = 0;j < OutOvlFiles[1].Types[i].count;j++)
					{
						fwrite(&OutOvlFiles[1].Types[i].dataptr[j].size,sizeof(OutOvlFiles[1].Types[i].dataptr[j].size),1,f);
						fwrite(OutOvlFiles[1].Types[i].dataptr[j].data,OutOvlFiles[1].Types[i].dataptr[j].size,1,f);
					}
				}
				unsigned long size = fixups.size();
				fwrite(&size,sizeof(size),1,f);
				while (fixups.empty() == false)
				{
					unsigned long fixup = fixups.front();
					fixups.pop();
					fwrite(&fixup,sizeof(fixup),1,f);
				}
				int t1 = 4;
				int t2 = 1;
				for (i = 0;i < TextStrings.size();i++)
				{
					fwrite(&t1,4,1,f);
					fwrite(&t2,4,1,f);
				}
				for (i = 0;i < IconTextures.size();i++)
				{
					FILE *tex = fopen(IconTextures[i]->filename,"rb");
					if (tex == 0)
					{
						MessageBox(hwnd,"Icon Texture not found","Error",MB_OK);
						return false;
					}
					fread(&bmf,sizeof(bmf),1,tex);
					fread(&bmi,sizeof(bmi),1,tex);
					if ((bmi.biWidth != bmi.biHeight) || (bmi.biBitCount != 24) || (bmi.biWidth != 512))
					{
						MessageBox(SceneryDlgHwnd,"Icon Texture not true color 512x512 texture ","Error",MB_OK);
						return false;
					}
					data = new unsigned char[bmi.biWidth * bmi.biHeight * 4];
					unsigned char *temp;
					temp = data;
					for (long i = 1;i <= bmi.biHeight;i++)
					{
						for (long j = 0;j < bmi.biWidth;j++)
						{
							*(temp+3) = 0;
							fread(temp,3,1,tex);
							temp+=4;
						}
					}
					fclose(tex);
					unsigned long size = sizeof(FlicHeader) + sizeof(FlicMipHeader) + (512 * 512 * 4) + sizeof(FlicMipHeader);
					fwrite(&size,4,1,f);
					FlicHeader fh;
					FlicMipHeader fmh;
					fh.Format = 3;
					fh.Height = 512;
					fh.Width = 512;
					fh.Mipcount = 0;
					fmh.Blocks = 512;
					fmh.MHeight = 512;
					fmh.MWidth = 512;
					fmh.Pitch = 2048;
					fwrite(&fh,sizeof(FlicHeader),1,f);
					fwrite(&fmh,sizeof(FlicMipHeader),1,f);
					fwrite(data,(512 * 512 * 4),1,f);
					fmh.Blocks = 0;
					fmh.MHeight = 0;
					fmh.MWidth = 0;
					fmh.Pitch = 0;
					fwrite(&fmh,sizeof(FlicMipHeader),1,f);
				}
				fclose(f);
				h.magic = 0x4B524746;
				h.References = 0;
				h.reserved = 0;
				h.version = 1;
				h2.FileTypeCount = 5;
				h2.unk = 0;
				len1 = 4;
				str1 = "FGDK";
				len2 = 4;
				str2 = "Text";
				type = 2;
				len3 = 3;
				str3 = "txt";
				strcpy(strstr(OutOvlFiles[1].FileName,".unique.ovl"),".common.ovl");
				strcpy(OutOvlFiles[0].FileName,OutOvlFiles[1].FileName);
				f = fopen(OutOvlFiles[0].FileName,"wb");
				fwrite(&h,sizeof(h),1,f);
				fwrite(&h2,sizeof(h2),1,f);
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "FGDK";
				len2 = 7;
				str2 = "Texture";
				type = 2;
				len3 = 3;
				str3 = "tex";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "FGDK";
				len2 = 4;
				str2 = "Flic";
				type = 2;
				len3 = 4;
				str3 = "flic";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "FGDK";
				len2 = 0xB;
				str2 = "GUISkinItem";
				type = 1;
				len3 = 3;
				str3 = "gsi";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				len1 = 4;
				str1 = "RCT3";
				len2 = 0xB;
				str2 = "SceneryItem";
				type = 1;
				len3 = 3;
				str3 = "sid";
				fwrite(&len1,sizeof(len1),1,f);
				fwrite(str1,len1,1,f);
				fwrite(&len2,sizeof(len2),1,f);
				fwrite(str2,len2,1,f);
				fwrite(&type,sizeof(type),1,f);
				fwrite(&len3,sizeof(len3),1,f);
				fwrite(str3,len3,1,f);
				unsigned long tmp = 1;
				fwrite(&tmp,4,1,f);
				tmp = 0;
				for (i = 0;i < 8;i++)
				{
					fwrite(&tmp,4,1,f);
				}
				fwrite(&Type0Files[0].size,4,1,f);
				fwrite(Type0Files[0].data,Type0Files[0].size,1,f);
				fwrite(&tmp,4,1,f);
				fclose(f);
				MessageBox(hwnd,"Theme successfully installed.","Completed",MB_OK); //added by KLN
				delete data;
				delete sidsx;
				delete gsis;
				delete textures;
				delete flics;
				delete textstrings;
				delete symrefs;
				delete loaders;
				delete symbols;
				delete stringtable;
				delete theme;
			}
			break;
		case ID_EDIT_CREATESCENERY:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_CREATESCENERY), hwnd, CreateSceneryDlgProc);
			break;
		case ID_EDIT_MANAGESCENERY:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_SIDMAN), hwnd, SIDManDlgProc);
			break;
		case ID_EDIT_MANAGETEXTURES:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTUREMAN), hwnd, TextureManDlgProc);
			break;
		case ID_EDIT_MANAGEREFERENCES:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIMAN), hwnd, GSIManDlgProc);
			break;
		case ID_EDIT_MANAGESTRINGS:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTMAN), hwnd, TextManDlgProc);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	hInst = hInstance;
	DialogBox(hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDlgProc);
	for (unsigned long i = 0;i < SceneryItems.size();i++)
	{
		delete SceneryItems[i]->name;
		delete SceneryItems[i]->ovl;
		delete SceneryItems[i]->location;
		delete SceneryItems[i]->icon;
		if (SceneryItems[i]->wallicon != 0)
		{
			delete SceneryItems[i]->wallicon;
			delete SceneryItems[i]->wallname;
		}
		delete SceneryItems[i];
	}
	for (i = 0;i < IconTextures.size();i++)
	{
		delete IconTextures[i]->filename;
		delete IconTextures[i]->name;
		delete IconTextures[i];
	}
	for (i = 0;i < Icons.size();i++)
	{
		delete Icons[i]->name;
		delete Icons[i]->texture;
		delete Icons[i];
	}
	for (i = 0;i < TextStrings.size();i++)
	{
		delete TextStrings[i]->name;
		delete TextStrings[i]->text;
		delete TextStrings[i];
	}
}
